from dataclasses import dataclass
from logging import getLogger

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QDialog

from custom_legendary.core import LegendaryCore
from rare.components.dialogs.login.browser_login import BrowserLogin
from rare.components.dialogs.login.import_login import ImportLogin
from rare.ui.components.dialogs.login.login_dialog import Ui_LoginDialog

logger = getLogger("Login")


@dataclass
class LoginPages:
    landing: int
    browser: int
    import_egl: int
    success: int


class LoginDialog(QDialog, Ui_LoginDialog):
    logged_in: bool = False
    pages = LoginPages(landing=0, browser=1, import_egl=2, success=3)

    def __init__(self, core: LegendaryCore, parent=None):
        super(LoginDialog, self).__init__(parent=parent)
        self.setupUi(self)
        self.setAttribute(Qt.WA_DeleteOnClose, True)
        self.setWindowFlags(Qt.Dialog | Qt.CustomizeWindowHint | Qt.WindowTitleHint)

        self.core = core

        self.browser_page = BrowserLogin(self.core, self.login_stack)
        self.login_stack.insertWidget(self.pages.browser, self.browser_page)
        self.browser_page.success.connect(self.login_successful)
        self.browser_page.changed.connect(
            lambda: self.next_button.setEnabled(self.browser_page.is_valid())
        )
        self.import_page = ImportLogin(self.core, self.login_stack)
        self.login_stack.insertWidget(self.pages.import_egl, self.import_page)
        self.import_page.success.connect(self.login_successful)
        self.import_page.changed.connect(
            lambda: self.next_button.setEnabled(self.import_page.is_valid())
        )

        self.next_button.setEnabled(False)
        self.back_button.setEnabled(False)

        self.login_browser_radio.clicked.connect(lambda: self.next_button.setEnabled(True))
        self.login_import_radio.clicked.connect(lambda: self.next_button.setEnabled(True))
        self.exit_button.clicked.connect(self.close)
        self.back_button.clicked.connect(self.back_clicked)
        self.next_button.clicked.connect(self.next_clicked)

        self.login_stack.setCurrentIndex(self.pages.landing)

        self.resize(self.minimumSizeHint())
        self.setFixedSize(self.size())

    def back_clicked(self):
        self.back_button.setEnabled(False)
        self.next_button.setEnabled(True)
        self.login_stack.setCurrentIndex(self.pages.landing)

    def next_clicked(self):
        if self.login_stack.currentIndex() == self.pages.landing:
            if self.login_browser_radio.isChecked():
                self.login_stack.setCurrentIndex(self.pages.browser)
                self.next_button.setEnabled(False)
            if self.login_import_radio.isChecked():
                self.login_stack.setCurrentIndex(self.pages.import_egl)
                self.next_button.setEnabled(self.import_page.is_valid())
            self.back_button.setEnabled(True)
        elif self.login_stack.currentIndex() == self.pages.browser:
            self.browser_page.do_login()
        elif self.login_stack.currentIndex() == self.pages.import_egl:
            self.import_page.do_login()
        else:
            self.close()

    def login(self):
        self.exec_()
        return self.logged_in

    def login_successful(self):
        try:
            if self.core.login():
                self.logged_in = True
                self.welcome_label.setText(
                    self.welcome_label.text().replace("</h1>", f", {self.core.lgd.userdata['displayName']}</h1>")
                )
                self.exit_button.setVisible(False)
                self.back_button.setVisible(False)
                self.login_stack.setCurrentIndex(self.pages.success)
            else:
                raise ValueError("Login failed.")
        except ValueError as e:
            logger.error(str(e))
            self.next_button.setEnabled(False)
            self.logged_in = False



import os
from logging import getLogger

from PyQt5.QtCore import Qt, QSettings, QTimer, pyqtSignal
from PyQt5.QtGui import QCloseEvent
from PyQt5.QtWidgets import QMainWindow, QMessageBox, QApplication

from custom_legendary.core import LegendaryCore
from rare.components.tab_widget import TabWidget
from rare.utils.rpc import DiscordRPC

logger = getLogger("Window")


class MainWindow(QMainWindow):
    quit_app = pyqtSignal(int)

    def __init__(self, core: LegendaryCore, args):
        super(MainWindow, self).__init__()
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.settings = QSettings()
        self.core = core
        self.offline = args.offline
        width, height = 1200, 800
        if self.settings.value("save_size", False):
            width, height = self.settings.value("window_size", (1200, 800), tuple)

        desktop = QApplication.desktop()
        self.setGeometry((desktop.width() - width) / 2, (desktop.height() - height) / 2, width, height)

        self.setWindowTitle("Rare - GUI for legendary")
        self.tab_widget = TabWidget(core, self, args.offline)
        self.tab_widget.quit_app.connect(self.quit_app.emit)
        self.setCentralWidget(self.tab_widget)
        if not args.offline:
            self.rpc = DiscordRPC(core)
            # Discord RPC on game launch
            self.tab_widget.games_tab.default_widget.game_list.game_started.connect(
                lambda: self.rpc.set_discord_rpc(self.tab_widget.games_tab.default_widget.game_list.running_games[0]))
            # Remove RPC
            self.tab_widget.delete_presence.connect(self.rpc.set_discord_rpc)
            # Show RPC on changed rare_settings
            self.tab_widget.settings.rare_settings.rpc.update_settings.connect(
                lambda: self.rpc.changed_settings(self.tab_widget.games_tab.default_widget.game_list.running_games))

        game = self.tab_widget.games_tab.default_widget.game_list.active_game
        if game != ("", 0) and not args.offline:
            self.rpc.set_discord_rpc(game[0])  # Appname

        if args.subparser == "launch":
            logger.info("Launching " + self.core.get_installed_game(args.app_name).title)
            if args.app_name in self.tab_widget.games_tab.default_widget.game_list.widgets.keys():
                self.tab_widget.games_tab.default_widget.game_list.widgets[args.app_name][1].launch()
            else:
                logger.info(f"Could not find {args.app_name} in Games")

        self.timer = QTimer()
        self.timer.timeout.connect(self.timer_finished)
        self.timer.start(1000)

    def timer_finished(self):
        file_path = os.path.expanduser("~/.cache/rare/lockfile")
        if os.path.exists(file_path):
            file = open(file_path, "r")
            action = file.read()
            file.close()
            if action.startswith("launch"):
                game = action.replace("launch ", "").replace("\n", "")
                if game in self.tab_widget.games_tab.default_widget.game_list.widgets.keys():
                    self.tab_widget.games_tab.default_widget.game_list.widgets[game][1].launch()
                else:
                    logger.info(f"Could not find {game} in Games")

            elif action.startswith("start"):
                self.show()
            os.remove(file_path)
        self.timer.start(1000)

    def closeEvent(self, e: QCloseEvent):
        if self.settings.value("sys_tray", True, bool):
            self.hide()
            e.ignore()
            return
        elif self.offline:
            pass
        elif self.tab_widget.downloadTab.active_game is not None:
            if not QMessageBox.question(self, "Close",
                                        self.tr("There is a download active. Do you really want to exit app?"),
                                        QMessageBox.Yes, QMessageBox.No) == QMessageBox.Yes:
                e.ignore()
                return
        if self.settings.value("save_size", False, bool):
            size = self.size().width(), self.size().height()
            self.settings.setValue("window_size", size)
        e.accept()



import configparser
import logging
import os
import platform
import sys
import time

from PyQt5.QtCore import QSettings, QTranslator
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QApplication, QSystemTrayIcon, QStyleFactory

from custom_legendary.core import LegendaryCore
from rare import languages_path, resources_path
from rare.components.dialogs.launch_dialog import LaunchDialog
from rare.components.main_window import MainWindow
from rare.components.tray_icon import TrayIcon
from rare.utils.utils import get_lang, load_color_scheme

start_time = time.strftime('%y-%m-%d--%H-%M')  # year-month-day-hour-minute
file_name = os.path.expanduser(f"~/.cache/rare/logs/Rare_{start_time}.log")
if not os.path.exists(os.path.dirname(file_name)):
    os.makedirs(os.path.dirname(file_name))

logging.basicConfig(
    format='[%(name)s] %(levelname)s: %(message)s',
    level=logging.INFO,
    filename=file_name,
)
logger = logging.getLogger("Rare")


class App(QApplication):
    def __init__(self, args):
        super(App, self).__init__(sys.argv)
        self.args = args  # add some options
        # init Legendary
        try:
            self.core = LegendaryCore()
        except configparser.MissingSectionHeaderError as e:
            logger.warning(f"Config is corrupt: {e}")
            if config_path := os.environ.get('XDG_CONFIG_HOME'):
                path = os.path.join(config_path, 'legendary')
            else:
                path = os.path.expanduser('~/.config/legendary')
            with open(os.path.join(path, "config.ini"), "w") as config_file:
                config_file.write("[Legendary]")
            self.core = LegendaryCore()
        if "Legendary" not in self.core.lgd.config.sections():
            self.core.lgd.config.add_section("Legendary")
            self.core.lgd.save_config()

        # workaround if egl sync enabled, but no programdata_path
        # programdata_path might be unset if logging in through the browser
        if self.core.egl_sync_enabled:
            if self.core.egl.programdata_path is None:
                self.core.lgd.config.remove_option("Legendary", "egl_sync")
                self.core.lgd.save_config()
            else:
                if not os.path.exists(self.core.egl.programdata_path):
                    self.core.lgd.config.remove_option("Legendary", "egl_sync")
                    self.core.lgd.save_config()

        # set Application name for settings
        self.mainwindow = None
        self.tray_icon = None
        self.launch_dialog = None
        self.setApplicationName("Rare")
        self.setOrganizationName("Rare")
        settings = QSettings()
        if platform.system() != "Windows":
            if args.disable_protondb:
                settings.setValue("disable_protondb", True)
            if args.enable_protondb:
                settings.remove("disable_protondb")

        # Translator
        self.translator = QTranslator()
        lang = settings.value("language", get_lang(), type=str)
        if os.path.exists(languages_path + lang + ".qm"):
            self.translator.load(languages_path + lang + ".qm")
            logger.info("Your language is supported: " + lang)
        elif not lang == "en":
            logger.info("Your language is not supported")
        self.installTranslator(self.translator)

        # Style
        self.setStyle(QStyleFactory.create("Fusion"))
        if settings.value("color_scheme", None) is None and settings.value("style_sheet", None) is None:
            settings.setValue("color_scheme", "")
            settings.setValue("style_sheet", "RareStyle")
        if color := settings.value("color_scheme", False):
            settings.setValue("style_sheet", "")
            custom_palette = load_color_scheme(os.path.join(resources_path, "colors", color + ".scheme"))
            if custom_palette is not None:
                self.setPalette(custom_palette)
        elif style := settings.value("style_sheet", False):
            settings.setValue("color_scheme", "")
            stylesheet = open(os.path.join(resources_path, "stylesheets", style, "stylesheet.qss")).read()
            style_resource_path = os.path.join(resources_path, "stylesheets", style, "")
            if os.name == "nt":
                style_resource_path = style_resource_path.replace('\\', '/')
            self.setStyleSheet(stylesheet.replace("@path@", style_resource_path))
            # lk: for qresources stylesheets, not an ideal solution for modability,
            # lk: too many extra steps and I don't like binary files in git, even as strings.
            # importlib.import_module("rare.resources.stylesheets." + style)
            # resource = QFile(f":/{style}/stylesheet.qss")
            # resource.open(QIODevice.ReadOnly)
            # self.setStyleSheet(QTextStream(resource).readAll())
        self.setWindowIcon(QIcon(os.path.join(resources_path, "images", "Rare.png")))

        # launch app
        self.launch_dialog = LaunchDialog(self.core, args.offline)
        self.launch_dialog.quit_app.connect(self.launch_dialog.close)
        self.launch_dialog.quit_app.connect(lambda ec: exit(ec))
        self.launch_dialog.start_app.connect(self.start_app)
        self.launch_dialog.start_app.connect(self.launch_dialog.close)

        if not args.silent or args.subparser == "launch":
            self.launch_dialog.login()

    def start_app(self, offline=False):
        self.args.offline = offline
        self.mainwindow = MainWindow(self.core, self.args)
        self.mainwindow.quit_app.connect(self.exit_app)
        self.tray_icon = TrayIcon(self)
        self.tray_icon.exit_action.triggered.connect(self.exit_app)
        self.tray_icon.start_rare.triggered.connect(self.mainwindow.show)
        self.tray_icon.activated.connect(self.tray)
        if not offline:
            self.mainwindow.tab_widget.downloadTab.finished.connect(lambda x: self.tray_icon.showMessage(
                self.tr("Download finished"),
                self.tr("Download finished. {} is playable now").format(self.core.get_game(x[1]).app_title),
                QSystemTrayIcon.Information, 4000) if x[0] else None)

        if not self.args.silent:
            self.mainwindow.show()

    def tray(self, reason):
        if reason == QSystemTrayIcon.DoubleClick:
            self.mainwindow.show()
            logger.info("Show App")

    def exit_app(self, exit_code=0):
        if self.tray_icon is not None:
            self.tray_icon.deleteLater()
        if self.mainwindow is not None:
            self.mainwindow.close()
        self.processEvents()
        self.exit(exit_code)


def start(args):
    while True:
        app = App(args)
        exit_code = app.exec_()
        # if not restart
        # restart app
        del app
        if exit_code != -133742:
            break



import os.path
import platform
from logging import getLogger

from PyQt5.QtWidgets import QFileDialog, QMessageBox, QStackedWidget, QVBoxLayout, QDialog, QCheckBox, QLabel, \
    QHBoxLayout, QPushButton, QGroupBox, QWidget

from custom_legendary.core import LegendaryCore
from rare.ui.components.tabs.settings.legendary import Ui_legendary_settings
from rare.utils.extra_widgets import PathEdit
from rare.utils.utils import get_size

logger = getLogger("LegendarySettings")


class LegendarySettings(QStackedWidget, Ui_legendary_settings):
    def __init__(self, core: LegendaryCore):
        super(LegendarySettings, self).__init__()
        self.setupUi(self)

        self.core = core

        # Default installation directory
        self.install_dir = PathEdit(core.get_default_install_dir(),
                                    file_type=QFileDialog.DirectoryOnly,
                                    save_func=self.save_path)
        self.layout_install_dir.addWidget(self.install_dir)

        # Max Workers
        max_workers = self.core.lgd.config["Legendary"].get("max_workers", fallback=0)
        self.max_worker_select.setValue(int(max_workers))
        self.max_worker_select.valueChanged.connect(self.max_worker_save)

        # Cleanup
        self.clean_button.clicked.connect(
            lambda: self.cleanup(False)
        )
        self.clean_button_without_manifests.clicked.connect(
            lambda: self.cleanup(True)
        )
        self.setCurrentIndex(0)

        self.back_button.clicked.connect(lambda: self.setCurrentIndex(0))

        self.path_info.setText(self.tr(r"EGL path is at C:\ProgramData\Epic\EpicGamesLauncher\Data\Manifests"))
        path = os.path.expanduser("~/")
        if self.core.egl.programdata_path:
            path = self.core.egl.programdata_path
        else:
            possible_wine_prefixes = [os.path.expanduser("~/.wine"),
                                      os.path.expanduser("~/Games/epic-games-store")]
            for i in possible_wine_prefixes:
                if os.path.exists(p := os.path.join(i, "drive_c/ProgramData/Epic/EpicGamesLauncher/Data/Manifests")):
                    path = p

        self.path_edit = PathEdit(path, QFileDialog.DirectoryOnly, save_func=self.save_egl_path)
        self.pathedit_placeholder.addWidget(self.path_edit)
        if platform.system() != "Windows":
            self.core.lgd.config.set("Legendary", "egl_programdata")
            self.core.egl.programdata_path = path

        self.importable_widgets = []
        self.exportable_widgets = []

        if self.core.egl_sync_enabled:
            self.sync_button.setText(self.tr("Disable sync"))
        else:
            self.sync_button.setText(self.tr("Enable Sync"))

        self.sync_button.clicked.connect(self.sync)

        self.enable_sync_button.clicked.connect(self.enable_sync)
        self.sync_once_button.clicked.connect(self.core.egl_sync)

    def enable_sync(self):
        if not self.core.egl.programdata_path:
            if os.path.exists(path := self.path_edit.text()):
                self.core.lgd.config.set("Legendary", "egl_programdata", path)
                self.core.lgd.save_config()
                self.core.egl.programdata_path = path

        self.core.lgd.config.set('Legendary', 'egl_sync', "true")
        self.core.egl_sync()
        self.core.lgd.save_config()
        self.sync_button.setText(self.tr("Disable Sync"))
        self.enable_sync_button.setDisabled(True)

    def export_all(self):
        for w in self.exportable_widgets:
            w.export_game()

    def import_all(self):
        for w in self.importable_widgets:
            w.import_game()

    def save_egl_path(self):
        self.core.lgd.config.set("Legendary", "egl_programdata", self.path_edit.text())
        self.core.egl.programdata_path = self.path_edit.text()
        self.core.lgd.save_config()
        self.update_egl_widget()

    def sync(self):
        if self.core.egl_sync_enabled:
            # disable sync
            info = DisableSyncDialog().get_information()
            if info[0] == 0:
                if info[1]:
                    self.core.lgd.config.remove_option('Legendary', 'egl_sync')
                else:
                    self.core.lgd.config.remove_option('Legendary', 'egl_programdata')
                    self.core.lgd.config.remove_option('Legendary', 'egl_sync')
                    # remove EGL GUIDs from all games, DO NOT remove .egstore folders because that would fuck things up.
                    for igame in self.core.get_installed_list():
                        igame.egl_guid = ''
                        self.core.install_game(igame)
                self.core.lgd.save_config()
                self.sync_button.setText(self.tr("Enable Sync"))
        else:
            # enable sync
            self.enable_sync_button.setDisabled(False)
            self.update_egl_widget()
            self.setCurrentIndex(1)

    def update_egl_widget(self):
        self.exportable_widgets = []
        QWidget().setLayout(self.exportable_games.layout())
        QWidget().setLayout(self.importable_games.layout())
        importable_layout = QVBoxLayout()
        self.importable_games.setLayout(importable_layout)
        exportable_layout = QVBoxLayout()
        self.exportable_games.setLayout(exportable_layout)
        if not self.core.egl.programdata_path:
            self.importable_games.setVisible(False)
            self.exportable_games.setVisible(False)
            self.export_all_button.setVisible(False)
            self.import_all_button.setVisible(False)
            return

        self.importable_games.setVisible(True)
        self.exportable_games.setVisible(True)
        self.export_all_button.setVisible(True)
        self.import_all_button.setVisible(True)

        for igame in self.core.egl_get_exportable():
            w = EGLSyncWidget(igame, True, self.core)
            self.importable_widgets.append(w)
            self.exportable_games.layout().addWidget(w)
        if len(self.core.egl_get_exportable()) == 0:
            self.exportable_games.layout().addWidget(QLabel(self.tr("No games to export")))

        self.importable_widgets = []
        for game in self.core.egl_get_importable():
            w = EGLSyncWidget(game, False, self.core)
            self.importable_widgets.append(w)
            self.importable_games.layout().addWidget(w)
        if len(self.core.egl_get_importable()) == 0:
            self.importable_games.layout().addWidget(QLabel(self.tr("No games to import")))

    def save_path(self):
        self.core.lgd.config["Legendary"]["install_dir"] = self.install_dir.text()
        if self.install_dir.text() == "" and "install_dir" in self.core.lgd.config["Legendary"].keys():
            self.core.lgd.config["Legendary"].pop("install_dir")
        else:
            logger.info("Set config install_dir to " + self.install_dir.text())
        self.core.lgd.save_config()

    def max_worker_save(self, num_workers: str):
        if num_workers == "":
            self.core.lgd.config.remove_option("Legendary", "max_workers")
            self.core.lgd.save_config()
            return
        num_workers = int(num_workers)
        if num_workers == 0:
            self.core.lgd.config.remove_option("Legendary", "max_workers")
        else:
            self.core.lgd.config.set("Legendary", "max_workers", str(num_workers))
        self.core.lgd.save_config()

    def cleanup(self, keep_manifests):
        before = self.core.lgd.get_dir_size()
        logger.debug('Removing app metadata...')
        app_names = set(g.app_name for g in self.core.get_assets(update_assets=False))
        self.core.lgd.clean_metadata(app_names)

        if not keep_manifests:
            logger.debug('Removing manifests...')
            installed = [(ig.app_name, ig.version) for ig in self.core.get_installed_list()]
            installed.extend((ig.app_name, ig.version) for ig in self.core.get_installed_dlc_list())
            self.core.lgd.clean_manifests(installed)

        logger.debug('Removing tmp data')
        self.core.lgd.clean_tmp_data()

        after = self.core.lgd.get_dir_size()
        logger.info(f'Cleanup complete! Removed {(before - after) / 1024 / 1024:.02f} MiB.')
        if (before - after) > 0:
            QMessageBox.information(self, "Cleanup", self.tr("Cleanup complete! Successfully removed {}").format(
                get_size(before - after)))
        else:
            QMessageBox.information(self, "Cleanup", "Nothing to clean")


class DisableSyncDialog(QDialog):
    info = 1, False

    def __init__(self):
        super(DisableSyncDialog, self).__init__()
        self.layout = QVBoxLayout()

        self.question = QLabel(self.tr("Do you really want to disable sync with Epic Games Store"))
        self.layout.addWidget(self.question)

        self.remove_metadata = QCheckBox(self.tr("Remove metadata from installed games"))
        self.layout.addWidget(self.remove_metadata)

        self.button_layout = QHBoxLayout()
        self.button_layout.addStretch(1)

        self.ok_button = QPushButton(self.tr("Ok"))
        self.cancel_button = QPushButton(self.tr("Cancel"))

        self.ok_button.clicked.connect(self.ok)
        self.cancel_button.clicked.connect(self.cancel)

        self.button_layout.addWidget(self.ok_button)
        self.button_layout.addWidget(self.cancel_button)

        self.layout.addStretch(1)
        self.layout.addLayout(self.button_layout)

        self.setLayout(self.layout)

    def ok(self):
        self.info = 0, self.remove_metadata.isChecked()
        self.close()

    def cancel(self):
        self.close()

    def get_information(self):
        self.exec_()
        return self.info


class EGLSyncWidget(QGroupBox):
    def __init__(self, game, export: bool, core: LegendaryCore):
        super(EGLSyncWidget, self).__init__()
        self.layout = QVBoxLayout()
        self.export = export
        self.core = core
        self.game = game
        if export:
            self.app_title_label = QLabel(game.title)
        else:
            title = self.core.get_game(game.app_name).app_title
            self.app_title_label = QLabel(title)
        self.layout.addWidget(self.app_title_label)
        self.button = QPushButton(self.tr("Export") if export else self.tr("Import"))

        if export:
            self.button.clicked.connect(self.export_game)
        else:
            self.button.clicked.connect(self.import_game)

        self.layout.addWidget(self.button)
        self.setLayout(self.layout)

    def export_game(self):
        self.core.egl_export(self.game.app_name)

    def import_game(self):
        self.core.egl_import(self.game.app_name)



import os
import platform

from PyQt5.QtCore import QSettings
from PyQt5.QtWidgets import QWidget, QFileDialog, QMessageBox

from custom_legendary.core import LegendaryCore
from custom_legendary.models.game import InstalledGame, Game
from rare.components.tabs.settings.linux import LinuxSettings
from rare.ui.components.tabs.games.game_info.game_settings import Ui_GameSettings
from rare.utils.extra_widgets import PathEdit


def find_proton_wrappers():
    possible_proton_wrappers = []
    compatibilitytools_dirs = [
        os.path.expanduser("~/.steam/steam/steamapps/common"),
        "/usr/share/steam/compatibilitytools.d",
        os.path.expanduser("~/.steam/compatibilitytools.d"),
        os.path.expanduser("~/.steam/root/compatibilitytools.d")
    ]
    for c in compatibilitytools_dirs:
        if os.path.exists(c):
            for i in os.listdir(c):
                proton = os.path.join(c, i, "proton")
                compatibilitytool = os.path.join(c, i, "compatibilitytool.vdf")
                toolmanifest = os.path.join(c, i, "toolmanifest.vdf")
                if os.path.exists(proton) and (os.path.exists(compatibilitytool) or os.path.exists(toolmanifest)):
                    wrapper = '"' + proton + '" run'
                    possible_proton_wrappers.append(wrapper)
    if not possible_proton_wrappers:
        print("Unable to find any Proton version")
    return possible_proton_wrappers


class GameSettings(QWidget, Ui_GameSettings):
    game: Game
    igame: InstalledGame

    # variable to no update when changing game
    change = False

    def __init__(self, core: LegendaryCore, parent):
        super(GameSettings, self).__init__(parent=parent)
        self.setupUi(self)

        self.core = core
        self.settings = QSettings()
        self.offline.currentIndexChanged.connect(
            lambda x: self.update_combobox(x, "offline")
        )
        self.skip_update.currentIndexChanged.connect(
            lambda x: self.update_combobox(x, "skip_update_check")
        )
        self.cloud_sync.stateChanged.connect(
            lambda: self.settings.setValue(f"{self.game.app_name}/auto_sync_cloud", self.cloud_sync.isChecked())
        )
        self.launch_params.textChanged.connect(lambda: self.launch_params_button.setEnabled(True))
        self.launch_params_button.clicked.connect(
            lambda: self.save_line_edit("start_params", self.launch_params.text())
        )
        self.launch_params_button.setEnabled(False)
        self.wrapper.textChanged.connect(lambda: self.wrapper_button.setEnabled(True))
        self.wrapper_button.clicked.connect(
            lambda: self.save_line_edit("wrapper", self.wrapper.text())
        )
        self.wrapper_button.setEnabled(False)

        if platform.system() != "Windows":
            self.possible_proton_wrappers = find_proton_wrappers()

            self.proton_wrapper.addItems(self.possible_proton_wrappers)
            self.proton_wrapper.currentIndexChanged.connect(self.change_proton)

            self.proton_prefix = PathEdit("None", QFileDialog.DirectoryOnly, save_func=self.update_prefix)
            self.proton_prefix_layout.addWidget(self.proton_prefix)

            self.linux_settings = LinuxAppSettings(core)
            self.linux_layout.addWidget(self.linux_settings)
        else:
            self.proton_groupbox.setVisible(False)

        # startparams, skip_update_check

    def save_line_edit(self, option, value):
        if value:
            if self.game.app_name not in self.core.lgd.config.sections():
                self.core.lgd.config.add_section(self.game.app_name)
            self.core.lgd.config.set(self.game.app_name, option, value)
        else:
            if self.core.lgd.config.has_section(self.game.app_name) and self.core.lgd.config.get(
                    f"{self.game.app_name}", option, fallback=None) is not None:
                self.core.lgd.config.remove_option(self.game.app_name, option)
                if not self.core.lgd.config[self.game.app_name]:
                    self.core.lgd.config.remove_section(self.game.app_name)
        self.core.lgd.save_config()
        self.sender().setEnabled(False)

    def update_combobox(self, i, option):
        if self.change:
            # remove section
            if i:
                if self.game.app_name not in self.core.lgd.config.sections():
                    self.core.lgd.config.add_section(self.game.app_name)
                if i == 1:
                    self.core.lgd.config.set(self.game.app_name, option, "true")
                if i == 2:
                    self.core.lgd.config.set(self.game.app_name, option, "false")
            else:
                if self.game.app_name in self.core.lgd.config.sections():
                    if self.core.lgd.config.get(f"{self.game.app_name}", option, fallback=False):
                        self.core.lgd.config.remove_option(self.game.app_name, option)
                if not self.core.lgd.config[self.game.app_name]:
                    self.core.lgd.config.remove_section(self.game.app_name)
            self.core.lgd.save_config()

    def change_proton(self, i):
        if self.change:
            # Dont use Proton
            if i == 0:
                if f"{self.game.app_name}" in self.core.lgd.config.sections():
                    if self.core.lgd.config.get(f"{self.game.app_name}", "wrapper", fallback=False):
                        self.core.lgd.config.remove_option(self.game.app_name, "wrapper")
                    if self.core.lgd.config.get(f"{self.game.app_name}", "no_wine", fallback=False):
                        self.core.lgd.config.remove_option(self.game.app_name, "no_wine")
                    if not self.core.lgd.config[self.game.app_name]:
                        self.core.lgd.config.remove_section(self.game.app_name)
                if f"{self.game.app_name}.env" in self.core.lgd.config.sections():
                    if self.core.lgd.config.get(f"{self.game.app_name}.env", "STEAM_COMPAT_DATA_PATH", fallback=False):
                        self.core.lgd.config.remove_option(f"{self.game.app_name}.env", "STEAM_COMPAT_DATA_PATH")
                    if not self.core.lgd.config[self.game.app_name + ".env"]:
                        self.core.lgd.config.remove_section(self.game.app_name + ".env")
                self.proton_prefix.setEnabled(False)
                # lk: TODO: This has to be fixed properly.
                # lk: It happens because of the widget update. Mask it for now behind disabling the save button
                self.wrapper.setText(self.core.lgd.config.get(f"{self.game.app_name}", "wrapper", fallback=""))
                self.wrapper_button.setDisabled(True)
                self.wrapper_widget.setEnabled(True)
                self.linux_settings.wine_groupbox.setEnabled(True)
            else:
                self.proton_prefix.setEnabled(True)
                self.wrapper_widget.setEnabled(False)
                self.linux_settings.wine_groupbox.setEnabled(False)
                wrapper = self.possible_proton_wrappers[i - 1]
                if self.game.app_name not in self.core.lgd.config.sections():
                    self.core.lgd.config[self.game.app_name] = {}
                if self.game.app_name + ".env" not in self.core.lgd.config.sections():
                    self.core.lgd.config[self.game.app_name + ".env"] = {}
                self.core.lgd.config.set(self.game.app_name, "wrapper", wrapper)
                self.core.lgd.config.set(self.game.app_name, "no_wine", "true")
                self.core.lgd.config.set(self.game.app_name + ".env", "STEAM_COMPAT_DATA_PATH",
                                         os.path.expanduser("~/.proton"))
                self.proton_prefix.text_edit.setText(os.path.expanduser("~/.proton"))

                # Dont use Wine
                self.linux_settings.wine_exec.text_edit.setText("")
                self.linux_settings.save_setting(self.linux_settings.wine_exec, "wine_exec")
                self.linux_settings.wine_prefix.text_edit.setText("")
                self.linux_settings.save_setting(self.linux_settings.wine_prefix, "wine_prefix")

        self.core.lgd.save_config()

    def update_prefix(self):
        text = self.proton_prefix.text()
        if not text:
            text = os.path.expanduser("~/.proton")
            self.proton_prefix.text_edit.setText(text)
        if not os.path.exists(text):
            try:
                os.makedirs(text)
            except PermissionError:
                QMessageBox.warning(self, "Warning", self.tr("No permission to create folder"))
                text = os.path.expanduser("~/.proton")
                self.proton_prefix.text_edit.setText(text)

        self.core.lgd.config.set(self.game.app_name + ".env", "STEAM_COMPAT_DATA_PATH", text)
        self.core.lgd.save_config()

    def update_game(self, app_name):
        self.change = False
        self.game = self.core.get_game(app_name)
        self.igame = self.core.get_installed_game(app_name)

        if self.igame.can_run_offline:
            offline = self.core.lgd.config.get(self.game.app_name, "offline", fallback="unset")
            if offline == "true":
                self.offline.setCurrentIndex(1)
            elif offline == "false":
                self.offline.setCurrentIndex(2)
            else:
                self.offline.setCurrentIndex(0)

            self.offline.setEnabled(True)
        else:
            self.offline.setEnabled(False)

        skip_update = self.core.lgd.config.get(self.game.app_name, "skip_update_check", fallback="unset")
        if skip_update == "true":
            self.skip_update.setCurrentIndex(1)
        elif skip_update == "false":
            self.skip_update.setCurrentIndex(2)
        else:
            self.skip_update.setCurrentIndex(0)

        wrapper = self.core.lgd.config.get(self.game.app_name, "wrapper", fallback="")
        self.wrapper.setText(wrapper)

        self.title.setText(f"<h2>{self.game.app_title}</h2>")
        if platform.system() != "Windows":
            self.linux_settings.update_game(app_name)
            self.linux_settings.dxvk.update_settings(app_name)
            proton = self.core.lgd.config.get(f"{app_name}", "wrapper", fallback="").replace('"', "")
            if proton != "":
                self.proton_prefix.setEnabled(True)
                self.proton_wrapper.setCurrentText(f'"{proton.replace(" run", "")}" run')
                proton_prefix = self.core.lgd.config.get(f"{app_name}.env", "STEAM_COMPAT_DATA_PATH",
                                                         fallback=self.tr(
                                                             "Please select path for proton prefix"))
                self.proton_prefix.text_edit.setText(proton_prefix)
                self.wrapper_widget.setEnabled(False)
            else:
                self.proton_wrapper.setCurrentIndex(0)
                self.proton_prefix.setEnabled(False)
                self.wrapper_widget.setEnabled(True)

        if not self.game.supports_cloud_saves:
            self.cloud_sync.setEnabled(False)
        else:
            self.cloud_sync.setEnabled(True)
            sync_cloud = self.settings.value(f"{self.game.app_name}/auto_sync_cloud", True, bool)
            self.cloud_sync.setChecked(sync_cloud)

        self.launch_params.setText(self.core.lgd.config.get(self.game.app_name, "start_params", fallback=""))
        self.change = True


class LinuxAppSettings(LinuxSettings):
    def __init__(self, core):
        super(LinuxAppSettings, self).__init__(core, "app")

    def update_game(self, app_name):
        self.name = app_name
        self.wine_prefix.text_edit.setText(self.core.lgd.config.get(self.name, "wine_prefix", fallback=""))
        self.wine_exec.text_edit.setText(self.core.lgd.config.get(self.name, "wine_executable", fallback=""))
        self.dxvk.name = app_name
        self.dxvk.more_settings_widget.name = app_name



import json
import os
import platform
import shutil
import sys
from logging import getLogger

import requests
from PIL import Image, UnidentifiedImageError
from PyQt5.QtCore import pyqtSignal, QLocale, QSettings
from PyQt5.QtGui import QPalette, QColor, QPixmap

# Windows

if platform.system() == "Windows":
    from win32com.client import Dispatch

from rare import languages_path, resources_path, image_dir
# Mac not supported

from custom_legendary.core import LegendaryCore

logger = getLogger("Utils")
s = QSettings("Rare", "Rare")


def download_images(signal: pyqtSignal, core: LegendaryCore):
    if not os.path.isdir(image_dir):
        os.makedirs(image_dir)
        logger.info("Create Image dir")

    # Download Images
    games, dlcs = core.get_game_and_dlc_list()
    dlc_list = []
    for i in dlcs.values():
        dlc_list.append(i[0])
    game_list = games + dlc_list
    for i, game in enumerate(game_list):
        try:
            download_image(game)
        except json.decoder.JSONDecodeError:
            shutil.rmtree(f"{image_dir}/{game.app_name}")
            download_image(game)
        signal.emit(i / len(game_list) * 100)


def download_image(game, force=False):
    if force and os.path.exists(f"{image_dir}/{game.app_name}"):
        shutil.rmtree(f"{image_dir}/{game.app_name}")
    if not os.path.isdir(f"{image_dir}/" + game.app_name):
        os.mkdir(f"{image_dir}/" + game.app_name)

    # to git picture updates
    if not os.path.isfile(f"{image_dir}/{game.app_name}/image.json"):
        json_data = {"DieselGameBoxTall": None, "DieselGameBoxLogo": None, "Thumbnail": None}
    else:
        json_data = json.load(open(f"{image_dir}/{game.app_name}/image.json", "r"))
    # Download
    for image in game.metadata["keyImages"]:
        if image["type"] == "DieselGameBoxTall" or image["type"] == "DieselGameBoxLogo" or image["type"] == "Thumbnail":
            if image["type"] not in json_data.keys():
                json_data[image["type"]] = None
            if json_data[image["type"]] != image["md5"] or not os.path.isfile(
                    f"{image_dir}/{game.app_name}/{image['type']}.png"):
                # Download
                json_data[image["type"]] = image["md5"]
                # os.remove(f"{image_dir}/{game.app_name}/{image['type']}.png")
                json.dump(json_data, open(f"{image_dir}/{game.app_name}/image.json", "w"))
                logger.info(f"Download Image for Game: {game.app_title}")
                url = image["url"]
                with open(f"{image_dir}/{game.app_name}/{image['type']}.png", "wb") as f:
                    f.write(requests.get(url).content)
                    try:
                        img = Image.open(f"{image_dir}/{game.app_name}/{image['type']}.png")
                        img = img.resize((200, int(200 * 4 / 3)))
                        img.save(f"{image_dir}/{game.app_name}/{image['type']}.png")
                    except UnidentifiedImageError as e:
                        logger.warning(e)

    # scale and grey
    if not os.path.exists(os.path.join(image_dir, game.app_name + '/UninstalledArt.png')):

        if os.path.exists(os.path.join(image_dir, f"{game.app_name}/DieselGameBoxTall.png")):
            # finalArt = Image.open(f'{image_dir}/' + game.app_name + '/DieselGameBoxTall.png')
            # finalArt.save(f'{image_dir}/{game.app_name}/FinalArt.png')
            # And same with the grayscale one

            bg = Image.open(os.path.join(image_dir, f"{game.app_name}/DieselGameBoxTall.png"))
            uninstalledArt = bg.convert('L')
            uninstalledArt = uninstalledArt.resize((200, int(200 * 4 / 3)))
            uninstalledArt.save(f'{image_dir}/{game.app_name}/UninstalledArt.png')
        elif os.path.isfile(f"{image_dir}/{game.app_name}/DieselGameBoxLogo.png"):
            bg: Image.Image = Image.open(f"{image_dir}/{game.app_name}/DieselGameBoxLogo.png")
            bg = bg.resize((int(bg.size[1] * 3 / 4), bg.size[1]))
            logo = Image.open(f'{image_dir}/{game.app_name}/DieselGameBoxLogo.png').convert('RGBA')
            wpercent = ((bg.size[0] * (3 / 4)) / float(logo.size[0]))
            hsize = int((float(logo.size[1]) * float(wpercent)))
            logo = logo.resize((int(bg.size[0] * (3 / 4)), hsize), Image.ANTIALIAS)
            # Calculate where the image has to be placed
            pasteX = int((bg.size[0] - logo.size[0]) / 2)
            pasteY = int((bg.size[1] - logo.size[1]) / 2)
            # And finally copy the background and paste in the image
            # finalArt = bg.copy()
            # finalArt.paste(logo, (pasteX, pasteY), logo)
            # Write out the file
            # finalArt.save(f'{image_dir}/' + game.app_name + '/FinalArt.png')
            logoCopy = logo.copy()
            logoCopy.putalpha(int(256 * 3 / 4))
            logo.paste(logoCopy, logo)
            uninstalledArt = bg.copy()
            uninstalledArt.paste(logo, (pasteX, pasteY), logo)
            uninstalledArt = uninstalledArt.convert('L')
            uninstalledArt.save(f'{image_dir}/' + game.app_name + '/UninstalledArt.png')
        else:
            logger.warning(f"File {image_dir}/{game.app_name}/DieselGameBoxTall.png doesn't exist")


def get_lang():
    core = LegendaryCore()
    if "Legendary" in core.lgd.config.sections() and "locale" in core.lgd.config["Legendary"]:
        logger.info("Found locale in Legendary config: " + core.lgd.config.get("Legendary", "locale"))
        return core.lgd.config.get("Legendary", "locale").split("-")[0]
    else:
        logger.info("Found locale in system config: " + QLocale.system().name().split("_")[0])
        return QLocale.system().name().split("_")[0]


color_role_map = {
    0: "WindowText",
    1: "Button",
    2: "Light",
    3: "Midlight",
    4: "Dark",
    5: "Mid",
    6: "Text",
    7: "BrightText",
    8: "ButtonText",
    9: "Base",
    10: "Window",
    11: "Shadow",
    12: "Highlight",
    13: "HighlightedText",
    14: "Link",
    15: "LinkVisited",
    16: "AlternateBase",
    # 17: "NoRole",
    18: "ToolTipBase",
    19: "ToolTipText",
    20: "PlaceholderText",
    # 21: "NColorRoles",
}

color_group_map = {
    0: "Active",
    1: "Disabled",
    2: "Inactive",
}


def load_color_scheme(path: str):
    palette = QPalette()
    settings = QSettings(path, QSettings.IniFormat)
    try:
        settings.beginGroup("ColorScheme")
        for g in color_group_map:
            settings.beginGroup(color_group_map[g])
            group = QPalette.ColorGroup(g)
            for r in color_role_map:
                role = QPalette.ColorRole(r)
                color = settings.value(color_role_map[r], None)
                if color is not None:
                    palette.setColor(group, role, QColor(color))
                else:
                    palette.setColor(group, role, palette.color(QPalette.Active, role))
            settings.endGroup()
        settings.endGroup()
    except:
        palette = None
    return palette


def get_color_schemes():
    colors = []
    for file in os.listdir(os.path.join(resources_path, "colors")):
        if file.endswith(".scheme") and os.path.isfile(os.path.join(resources_path, "colors", file)):
            colors.append(file.replace(".scheme", ""))
    return colors


def get_style_sheets():
    styles = []
    for folder in os.listdir(os.path.join(resources_path, "stylesheets")):
        if os.path.isfile(os.path.join(resources_path, "stylesheets", folder, "stylesheet.qss")):
            styles.append(folder)
    return styles


def get_possible_langs():
    langs = ["en"]
    for i in os.listdir(languages_path):
        if i.endswith(".qm"):
            langs.append(i.split(".")[0])
    return langs


def get_latest_version():
    try:
        resp = requests.get("https://api.github.com/repos/Dummerle/Rare/releases/latest")
        tag = resp.json()["tag_name"]
        return tag
    except requests.exceptions.ConnectionError:
        return "0.0.0"


def get_size(b: int) -> str:
    for i in ['', 'K', 'M', 'G', 'T', 'P', 'E']:
        if b < 1024:
            return f"{b:.2f}{i}B"
        b /= 1024


def create_rare_desktop_link(type_of_link):
    # Linux
    if platform.system() == "Linux":
        if type_of_link == "desktop":
            path = os.path.expanduser("~/Desktop/")
        elif type_of_link == "start_menu":
            path = os.path.expanduser("~/.local/share/applications/")
        else:
            return

        if p := os.environ.get("APPIMAGE"):
            executable = p
        else:
            executable = f"{sys.executable} {os.path.abspath(sys.argv[0])}"
        with open(os.path.join(path, "Rare.desktop"), "w") as desktop_file:
            desktop_file.write("[Desktop Entry]\n"
                               f"Name=Rare\n"
                               f"Type=Application\n"
                               f"Icon={os.path.join(resources_path, 'images', 'Rare.png')}\n"
                               f"Exec={executable}\n"
                               "Terminal=false\n"
                               "StartupWMClass=rare\n"
                               )
            desktop_file.close()
        os.chmod(os.path.expanduser(os.path.join(path, "Rare.desktop")), 0o755)

    elif platform.system() == "Windows":
        # Target of shortcut
        if type_of_link == "desktop":
            target_folder = os.path.expanduser('~/Desktop/')
        elif type_of_link == "start_menu":
            target_folder = os.path.expandvars("%appdata%/Microsoft/Windows/Start Menu")
        else:
            logger.warning("No valid type of link")
            return
        linkName = "Rare.lnk"

        # Path to location of link file
        pathLink = os.path.join(target_folder, linkName)

        exexutable = sys.executable
        if "python.exe" in exexutable:
            exexutable = exexutable.replace("python.exe", "pythonw.exe")

        # Add shortcut
        shell = Dispatch('WScript.Shell')
        shortcut = shell.CreateShortCut(pathLink)
        shortcut.Targetpath = exexutable
        shortcut.Arguments = os.path.abspath(sys.argv[0])
        shortcut.WorkingDirectory = os.getcwd()

        # Icon
        shortcut.IconLocation = os.path.join(resources_path, "images", "Rare.ico")

        shortcut.save()


def create_desktop_link(app_name, core: LegendaryCore, type_of_link="desktop") -> bool:
    igame = core.get_installed_game(app_name)

    if os.path.exists(
            os.path.join(QSettings('Rare', 'Rare').value('img_dir', os.path.expanduser('~/.cache/rare/images'), str),
                         igame.app_name, 'Thumbnail.png')):
        icon = os.path.join(QSettings('Rare', 'Rare').value('img_dir', os.path.expanduser('~/.cache/rare/images'), str),
                            igame.app_name, 'Thumbnail')
    else:
        icon = os.path.join(QSettings('Rare', 'Rare').value('img_dir', os.path.expanduser('~/.cache/rare/images'), str),
                            igame.app_name, 'DieselGameBoxTall')
    # Linux
    if platform.system() == "Linux":
        if type_of_link == "desktop":
            path = os.path.expanduser(f"~/Desktop/")
        elif type_of_link == "start_menu":
            path = os.path.expanduser("~/.local/share/applications/")
        else:
            return False
        if not os.path.exists(path):
            return False
        if p := os.environ.get("APPIMAGE"):
            executable = p
        else:
            executable = f"{sys.executable} {os.path.abspath(sys.argv[0])}"
        with open(f"{path}{igame.title}.desktop", "w") as desktop_file:
            desktop_file.write("[Desktop Entry]\n"
                               f"Name={igame.title}\n"
                               f"Type=Application\n"
                               f"Icon={icon}.png\n"
                               f"Exec={executable} launch {app_name}\n"
                               "Terminal=false\n"
                               "StartupWMClass=rare-game\n"
                               )
            desktop_file.close()
        os.chmod(os.path.expanduser(f"{path}{igame.title}.desktop"), 0o755)

    # Windows
    elif platform.system() == "Windows":
        # Target of shortcut
        if type_of_link == "desktop":
            target_folder = os.path.expanduser('~/Desktop/')
        elif type_of_link == "start_menu":
            target_folder = os.path.expandvars("%appdata%/Microsoft/Windows/Start Menu")
        else:
            logger.warning("No valid type of link")
            return False
        if not os.path.exists(target_folder):
            return False

        target = os.path.abspath(sys.argv[0])

        # Name of link file

        linkName = igame.title
        for c in r'<>?":|\/*':
            linkName.replace(c, "")

        linkName = linkName.strip() + '.lnk'

        # Path to location of link file
        pathLink = os.path.join(target_folder, linkName)

        # Add shortcut
        shell = Dispatch('WScript.Shell')
        shortcut = shell.CreateShortCut(pathLink)
        shortcut.Targetpath = sys.executable
        shortcut.Arguments = f'{target} launch {app_name}'
        shortcut.WorkingDirectory = os.getcwd()

        # Icon
        if not os.path.exists(icon + ".ico"):
            img = Image.open(icon + ".png")
            img.save(icon + ".ico")
            logger.info("Create Icon")
        shortcut.IconLocation = os.path.join(icon + ".ico")

        shortcut.save()
        return True

    elif platform.system() == "Darwin":
        return False


def get_pixmap(app_name: str) -> QPixmap:
    for img in ["FinalArt.png", "DieselGameBoxTall.png", "DieselGameBoxLogo.png"]:
        if os.path.exists(image := os.path.join(image_dir, app_name, img)):
            pixmap = QPixmap(image)
            break
    else:
        pixmap = QPixmap()
    return pixmap


def get_uninstalled_pixmap(app_name: str) -> QPixmap:
    if os.path.exists(image := os.path.join(image_dir, app_name, "UninstalledArt.png")):
        pixmap = QPixmap(image)
    else:
        pixmap = QPixmap()
    return pixmap



import os
from dataclasses import field, dataclass
from multiprocessing import Queue

from custom_legendary.downloader.manager import DLManager
from custom_legendary.models.downloading import AnalysisResult, ConditionCheckResult
from custom_legendary.models.game import Game, InstalledGame


@dataclass
class InstallOptionsModel:
    app_name: str
    base_path: str = os.path.expanduser("~/legendary")
    max_workers: int = os.cpu_count() * 2
    repair: bool = False
    no_install: bool = False
    ignore_space_req: bool = False
    force: bool = False
    sdl_list: list[str] = field(default_factory=lambda: [''])


@dataclass
class InstallDownloadModel:
    dlmanager: DLManager
    analysis: AnalysisResult
    game: Game
    igame: InstalledGame
    repair: bool
    repair_file: str
    res: ConditionCheckResult


@dataclass
class InstallQueueItemModel:
    status_q: Queue = None
    download: InstallDownloadModel = None
    options: InstallOptionsModel = None

    def __bool__(self):
        return (self.status_q is not None) and (self.download is not None) and (self.options is not None)



from PyQt5.QtGui import QTextCursor
from PyQt5.QtWidgets import QPlainTextEdit, QWidget, QPushButton, QFileDialog, QVBoxLayout


class ConsoleWindow(QWidget):
    def __init__(self):
        super(ConsoleWindow, self).__init__()
        self.layout = QVBoxLayout()
        self.setGeometry(0, 0, 600, 400)

        self.console = Console()
        self.layout.addWidget(self.console)

        self.save_button = QPushButton(self.tr("Save output to file"))
        self.layout.addWidget(self.save_button)
        self.save_button.clicked.connect(self.save)

        self.setLayout(self.layout)

    def save(self):
        file, ok = QFileDialog.getSaveFileName(self, "Save output", "", "Log Files (*.log);;All Files (*)")
        if ok:
            if "." not in file:
                file += ".log"
            with open(file, "w") as f:
                f.write(self.console.toPlainText())
                f.close()
                self.save_button.setText(self.tr("Saved"))

    def log(self, text):
        self.console.log(text)

    def error(self, text):
        self.console.error(text)


class Console(QPlainTextEdit):

    def __init__(self):
        super().__init__()
        self.setReadOnly(True)
        self._cursor_output = self.textCursor()

    def log(self, text):
        self._cursor_output.insertText(text)
        self.scroll_to_last_line()

    def error(self, text):
        self._cursor_output.insertHtml(f"<font color=\"Red\">{text}</font>")
        self.scroll_to_last_line()

    def scroll_to_last_line(self):
        cursor = self.textCursor()
        cursor.movePosition(QTextCursor.End)
        cursor.movePosition(QTextCursor.Up if cursor.atBlockStart() else
                            QTextCursor.StartOfLine)
        self.setTextCursor(cursor)



import os
import platform
import queue
import subprocess
import sys
import time
from logging import getLogger
from queue import Empty

import psutil
from PyQt5.QtCore import QThread, pyqtSignal
from PyQt5.QtWidgets import QMessageBox

from custom_legendary.core import LegendaryCore
from custom_legendary.models.downloading import UIUpdate, WriterTask
from rare.utils.models import InstallQueueItemModel

logger = getLogger("Download")


class DownloadThread(QThread):
    status = pyqtSignal(str)
    statistics = pyqtSignal(UIUpdate)

    def __init__(self, core: LegendaryCore, queue_item: InstallQueueItemModel):
        super(DownloadThread, self).__init__()
        self.core = core
        self.dlm = queue_item.download.dlmanager
        self.no_install = queue_item.options.no_install
        self.status_q = queue_item.status_q
        self.igame = queue_item.download.igame
        self.repair = queue_item.download.repair
        self.repair_file = queue_item.download.repair_file
        self._kill = False

    def run(self):
        start_time = time.time()
        dl_stopped = False
        try:

            self.dlm.start()
            time.sleep(1)
            while self.dlm.is_alive():
                if self._kill:
                    self.status.emit("stop")
                    logger.info("Download stopping...")

                    # The code below is a temporary solution.
                    # It should be removed once legendary supports stopping downloads more gracefully.

                    self.dlm.running = False

                    # send conditions to unlock threads if they aren't already
                    for cond in self.dlm.conditions:
                        with cond:
                            cond.notify()

                    # make sure threads are dead.
                    for t in self.dlm.threads:
                        t.join(timeout=5.0)
                        if t.is_alive():
                            logger.warning(f'Thread did not terminate! {repr(t)}')

                    # clean up all the queues, otherwise this process won't terminate properly
                    for name, q in zip(('Download jobs', 'Writer jobs', 'Download results', 'Writer results'),
                                       (self.dlm.dl_worker_queue, self.dlm.writer_queue, self.dlm.dl_result_q,
                                        self.dlm.writer_result_q)):
                        logger.debug(f'Cleaning up queue "{name}"')
                        try:
                            while True:
                                _ = q.get_nowait()
                        except Empty:
                            q.close()
                            q.join_thread()
                        except AttributeError:
                            logger.warning(f'Queue {name} did not close')

                    if self.dlm.writer_queue:
                        # cancel installation
                        self.dlm.writer_queue.put_nowait(WriterTask('', kill=True))

                    # forcibly kill DL workers that are not actually dead yet
                    for child in self.dlm.children:
                        if child.exitcode is None:
                            child.terminate()

                    if self.dlm.shared_memory:
                        # close up shared memory
                        self.dlm.shared_memory.close()
                        self.dlm.shared_memory.unlink()
                        self.dlm.shared_memory = None

                    self.dlm.kill()

                    # force kill any threads that are somehow still alive
                    for proc in psutil.process_iter():
                        # check whether the process name matches
                        if sys.platform in ['linux', 'darwin'] and proc.name() == 'DownloadThread':
                            proc.kill()
                        elif sys.platform == 'win32' and proc.name() == 'python.exe' and proc.create_time() >= start_time:
                            proc.kill()

                    logger.info("Download stopped. It can be continued later.")
                    dl_stopped = True
                try:
                    if not dl_stopped:
                        self.statistics.emit(self.status_q.get(timeout=1))
                except queue.Empty:
                    pass

            self.dlm.join()

        except Exception as e:
            logger.error(f"Installation failed after {time.time() - start_time:.02f} seconds: {e}")
            self.status.emit("error " + str(e))
            return

        else:
            if dl_stopped:
                return
            self.status.emit("dl_finished")
            end_t = time.time()
            logger.info(f"Download finished in {start_time - end_t}s")
            game = self.core.get_game(self.igame.app_name)

            if not self.no_install:
                postinstall = self.core.install_game(self.igame)
                if postinstall:
                    self._handle_postinstall(postinstall, self.igame)

                dlcs = self.core.get_dlc_for_game(self.igame.app_name)
                if dlcs:
                    print('The following DLCs are available for this game:')
                    for dlc in dlcs:
                        print(f' - {dlc.app_title} (App name: {dlc.app_name}, version: {dlc.app_version})')
                    print('Manually installing DLCs works the same; just use the DLC app name instead.')

                    # install_dlcs = QMessageBox.question(self, "", "Do you want to install the prequisites", QMessageBox.Yes|QMessageBox.No) == QMessageBox.Yes
                    # TODO
                if game.supports_cloud_saves and not game.is_dlc:
                    logger.info('This game supports cloud saves, syncing is handled by the "sync-saves" command.')
                    logger.info(f'To download saves for this game run "legendary sync-saves {game.app_name}"')
        old_igame = self.core.get_installed_game(game.app_name)
        if old_igame and self.repair and os.path.exists(self.repair_file):
            if old_igame.needs_verification:
                old_igame.needs_verification = False
                self.core.install_game(old_igame)

            logger.debug('Removing repair file.')
            os.remove(self.repair_file)
        if old_igame and old_igame.install_tags != self.igame.install_tags:
            old_igame.install_tags = self.igame.install_tags
            self.logger.info('Deleting now untagged files.')
            self.core.uninstall_tag(old_igame)
            self.core.install_game(old_igame)

        self.status.emit("finish")

    def _handle_postinstall(self, postinstall, igame):
        print('This game lists the following prequisites to be installed:')
        print(f'- {postinstall["name"]}: {" ".join((postinstall["path"], postinstall["args"]))}')
        if platform.system() == "Windows":
            if QMessageBox.question(self, "", "Do you want to install the prequisites",
                                    QMessageBox.Yes | QMessageBox.No) == QMessageBox.Yes:
                self.core.prereq_installed(igame.app_name)
                req_path, req_exec = os.path.split(postinstall['path'])
                work_dir = os.path.join(igame.install_path, req_path)
                fullpath = os.path.join(work_dir, req_exec)
                subprocess.call([fullpath, postinstall['args']], cwd=work_dir)
            else:
                self.core.prereq_installed(self.igame.app_name)

        else:
            logger.info('Automatic installation not available on Linux.')

    def kill(self):
        self._kill = True



import json
import os
from logging import getLogger

from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QWidget, QLabel, QHBoxLayout, QPushButton, QVBoxLayout, QFileDialog, QLineEdit, \
    QGroupBox
from qtawesome import icon

from custom_legendary.core import LegendaryCore
from rare.utils import legendary_utils
from rare.utils.extra_widgets import PathEdit

logger = getLogger("Import")


class ImportWidget(QWidget):
    update_list = pyqtSignal(str)

    def __init__(self, core: LegendaryCore, parent):
        super(ImportWidget, self).__init__(parent=parent)
        self.core = core
        self.game_list = [i.app_name for i in self.core.get_game_list()]

        self.main_layout = QHBoxLayout()
        self.back_button = QPushButton(icon("mdi.keyboard-backspace", color="white"), self.tr("Back"))
        self.right_layout = QVBoxLayout()
        self.right_layout.addWidget(self.back_button)
        self.right_layout.addStretch(1)
        self.main_layout.addLayout(self.right_layout)
        self.back_button.setFixedWidth(75)
        self.layout = QVBoxLayout()

        self.title = QLabel("<h2>Import Game</h2")
        self.layout.addWidget(self.title)

        # self.import_one_game = QLabel(f"<h3>{self.tr('Import existing game from Epic Games Launcher')}</h3>")
        self.import_one_game = QGroupBox(self.tr('Import existing game from Epic Games Launcher'))
        self.import_one_game.setObjectName("group")
        self.gb_layout = QVBoxLayout()

        self.import_game_info = QLabel(self.tr("Select path to game"))
        self.gb_layout.addWidget(self.import_game_info)

        self.override_app_name_label = QLabel(
            self.tr("Override app name (Only if the app could not find the app name)"))
        self.override_app_name_label.setWordWrap(True)
        self.app_name_input = QLineEdit()
        self.app_name_input.setFixedHeight(32)
        minilayout = QHBoxLayout()
        minilayout.addStretch(1)
        self.indicator_label = QLabel("")
        minilayout.addWidget(self.indicator_label)
        self.app_name_input.setLayout(minilayout)
        self.app_name_input.textChanged.connect(self.app_name_changed)

        self.path_edit = PathEdit(os.path.expanduser("~"), QFileDialog.DirectoryOnly, edit_func=self.path_changed)
        self.gb_layout.addWidget(self.path_edit)

        self.gb_layout.addWidget(self.override_app_name_label)
        self.gb_layout.addWidget(self.app_name_input)

        self.info_label = QLabel("")
        self.gb_layout.addWidget(self.info_label)
        self.import_button = QPushButton(self.tr("Import Game"))
        self.gb_layout.addWidget(self.import_button)
        self.import_button.clicked.connect(self.import_game)

        self.import_one_game.setLayout(self.gb_layout)

        self.layout.addWidget(self.import_one_game)

        self.auto_import = QLabel(
            f"<h4>{self.tr('To import games from Epic Games Store, please enable EGL Sync in legendary settings')}</h4>")
        self.layout.addWidget(self.auto_import)
        self.layout.addStretch(1)

        self.main_layout.addLayout(self.layout)
        # self.main_layout.addStretch(1)
        self.setLayout(self.main_layout)

    def app_name_changed(self, text):
        if text in self.game_list:
            self.indicator_label.setPixmap(icon("ei.ok-sign", color="green").pixmap(16, 16))
        else:
            self.indicator_label.setPixmap(icon("ei.remove-sign", color="red").pixmap(16, 16))

    def path_changed(self, path):
        if os.path.exists(path):
            if os.path.exists(os.path.join(path, ".egstore")):
                self.app_name_input.setText(self.find_app_name(path))

    def find_app_name(self, path):
        if not os.path.exists(os.path.join(path, ".egstore")):
            return None
        for i in os.listdir(os.path.join(path, ".egstore")):
            if i.endswith(".mancpn"):
                file = os.path.join(path, ".egstore", i)
                break
        else:
            logger.warning("File was not found")
            return None
        return json.load(open(file, "r"))["AppName"]

    def import_game(self, path=None):
        app_name = self.app_name_input.text()
        if not path:
            path = self.path_edit.text()
        if not app_name:
            # try to find app name
            if a_n := self.find_app_name(path):
                app_name = a_n
            else:
                self.info_label.setText(self.tr("Could not find app name"))
                return

        if legendary_utils.import_game(self.core, app_name=app_name, path=path):
            self.info_label.setText(self.tr("Successfully imported {}. Reload library").format(
                self.core.get_installed_game(app_name).title))
            self.app_name_input.setText("")

            self.update_list.emit(app_name)
        else:
            logger.warning("Failed to import" + app_name)
            self.info_label.setText(self.tr("Failed to import {}").format(app_name))
            return



import platform

from PyQt5.QtWidgets import QTabWidget

from rare.components.tabs.settings.about import About
from rare.components.tabs.settings.legendary import LegendarySettings
from rare.components.tabs.settings.linux import LinuxSettings
from rare.components.tabs.settings.rare import RareSettings
from rare.utils.extra_widgets import SideTabBar


class SettingsTab(QTabWidget):
    def __init__(self, core, parent):
        super(SettingsTab, self).__init__(parent=parent)
        self.core = core
        self.setTabBar(SideTabBar())
        self.setTabPosition(QTabWidget.West)
        self.rare_settings = RareSettings()
        self.addTab(self.rare_settings, "Rare")
        self.addTab(LegendarySettings(core), "Legendary")
        if platform.system() != "Windows":
            self.addTab(LinuxSettings(core), "Linux")
        self.about = About()

        self.addTab(self.about, "About (!)" if self.about.update_available else "About")



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'install_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_InstallDialog(object):
    def setupUi(self, InstallDialog):
        InstallDialog.setObjectName("InstallDialog")
        InstallDialog.resize(338, 267)
        InstallDialog.setWindowTitle("Rare")
        self.install_dialog_layout = QtWidgets.QGridLayout(InstallDialog)
        self.install_dialog_layout.setObjectName("install_dialog_layout")
        self.force_download_check = QtWidgets.QCheckBox(InstallDialog)
        self.force_download_check.setObjectName("force_download_check")
        self.install_dialog_layout.addWidget(self.force_download_check, 3, 1, 1, 1)
        self.install_dir_label = QtWidgets.QLabel(InstallDialog)
        self.install_dir_label.setObjectName("install_dir_label")
        self.install_dialog_layout.addWidget(self.install_dir_label, 1, 0, 1, 1, QtCore.Qt.AlignRight)
        self.download_only_label = QtWidgets.QLabel(InstallDialog)
        self.download_only_label.setObjectName("download_only_label")
        self.install_dialog_layout.addWidget(self.download_only_label, 5, 0, 1, 1, QtCore.Qt.AlignRight)
        self.sdl_list_label = QtWidgets.QLabel(InstallDialog)
        self.sdl_list_label.setObjectName("sdl_list_label")
        self.install_dialog_layout.addWidget(self.sdl_list_label, 6, 0, 1, 1, QtCore.Qt.AlignRight)
        self.install_size_info_label = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.install_size_info_label.setFont(font)
        self.install_size_info_label.setObjectName("install_size_info_label")
        self.install_dialog_layout.addWidget(self.install_size_info_label, 8, 1, 1, 2)
        self.install_dialog_label = QtWidgets.QLabel(InstallDialog)
        self.install_dialog_label.setObjectName("install_dialog_label")
        self.install_dialog_layout.addWidget(self.install_dialog_label, 0, 0, 1, 3)
        spacerItem = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.install_dialog_layout.addItem(spacerItem, 3, 2, 1, 1)
        self.install_size_label = QtWidgets.QLabel(InstallDialog)
        self.install_size_label.setObjectName("install_size_label")
        self.install_dialog_layout.addWidget(self.install_size_label, 8, 0, 1, 1, QtCore.Qt.AlignRight)
        self.download_only_info_label = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.download_only_info_label.setFont(font)
        self.download_only_info_label.setObjectName("download_only_info_label")
        self.install_dialog_layout.addWidget(self.download_only_info_label, 5, 2, 1, 1)
        self.max_workers_spin = QtWidgets.QSpinBox(InstallDialog)
        self.max_workers_spin.setObjectName("max_workers_spin")
        self.install_dialog_layout.addWidget(self.max_workers_spin, 2, 1, 1, 1)
        self.ignore_space_check = QtWidgets.QCheckBox(InstallDialog)
        self.ignore_space_check.setObjectName("ignore_space_check")
        self.install_dialog_layout.addWidget(self.ignore_space_check, 4, 1, 1, 1)
        self.download_only_check = QtWidgets.QCheckBox(InstallDialog)
        self.download_only_check.setText("")
        self.download_only_check.setObjectName("download_only_check")
        self.install_dialog_layout.addWidget(self.download_only_check, 5, 1, 1, 1)
        self.max_workers_info_label = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.max_workers_info_label.setFont(font)
        self.max_workers_info_label.setObjectName("max_workers_info_label")
        self.install_dialog_layout.addWidget(self.max_workers_info_label, 2, 2, 1, 1)
        self.warn_message = QtWidgets.QLabel(InstallDialog)
        self.warn_message.setWordWrap(True)
        self.warn_message.setObjectName("warn_message")
        self.install_dialog_layout.addWidget(self.warn_message, 9, 1, 1, 2)
        self.ignore_space_label = QtWidgets.QLabel(InstallDialog)
        self.ignore_space_label.setObjectName("ignore_space_label")
        self.install_dialog_layout.addWidget(self.ignore_space_label, 4, 0, 1, 1, QtCore.Qt.AlignRight)
        self.install_dir_layout = QtWidgets.QHBoxLayout()
        self.install_dir_layout.setObjectName("install_dir_layout")
        self.install_dialog_layout.addLayout(self.install_dir_layout, 1, 1, 1, 2)
        self.download_size_label = QtWidgets.QLabel(InstallDialog)
        self.download_size_label.setObjectName("download_size_label")
        self.install_dialog_layout.addWidget(self.download_size_label, 7, 0, 1, 1, QtCore.Qt.AlignRight)
        self.force_download_label = QtWidgets.QLabel(InstallDialog)
        self.force_download_label.setObjectName("force_download_label")
        self.install_dialog_layout.addWidget(self.force_download_label, 3, 0, 1, 1, QtCore.Qt.AlignRight)
        self.ignore_space_info_label = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.ignore_space_info_label.setFont(font)
        self.ignore_space_info_label.setObjectName("ignore_space_info_label")
        self.install_dialog_layout.addWidget(self.ignore_space_info_label, 4, 2, 1, 1)
        self.max_workers_label = QtWidgets.QLabel(InstallDialog)
        self.max_workers_label.setObjectName("max_workers_label")
        self.install_dialog_layout.addWidget(self.max_workers_label, 2, 0, 1, 1, QtCore.Qt.AlignRight)
        self.download_size_info_label = QtWidgets.QLabel(InstallDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.download_size_info_label.setFont(font)
        self.download_size_info_label.setObjectName("download_size_info_label")
        self.install_dialog_layout.addWidget(self.download_size_info_label, 7, 1, 1, 2)
        self.button_layout = QtWidgets.QHBoxLayout()
        self.button_layout.setObjectName("button_layout")
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.button_layout.addItem(spacerItem1)
        self.cancel_button = QtWidgets.QPushButton(InstallDialog)
        self.cancel_button.setObjectName("cancel_button")
        self.button_layout.addWidget(self.cancel_button)
        self.verify_button = QtWidgets.QPushButton(InstallDialog)
        self.verify_button.setObjectName("verify_button")
        self.button_layout.addWidget(self.verify_button)
        self.install_button = QtWidgets.QPushButton(InstallDialog)
        self.install_button.setObjectName("install_button")
        self.button_layout.addWidget(self.install_button)
        self.install_dialog_layout.addLayout(self.button_layout, 11, 0, 1, 3)
        self.sdl_list_frame = QtWidgets.QFrame(InstallDialog)
        self.sdl_list_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.sdl_list_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.sdl_list_frame.setObjectName("sdl_list_frame")
        self.sdl_list_layout = QtWidgets.QVBoxLayout(self.sdl_list_frame)
        self.sdl_list_layout.setSpacing(0)
        self.sdl_list_layout.setObjectName("sdl_list_layout")
        self.install_dialog_layout.addWidget(self.sdl_list_frame, 6, 1, 1, 2, QtCore.Qt.AlignTop)
        self.warn_label = QtWidgets.QLabel(InstallDialog)
        self.warn_label.setObjectName("warn_label")
        self.install_dialog_layout.addWidget(self.warn_label, 9, 0, 1, 1, QtCore.Qt.AlignRight)

        self.retranslateUi(InstallDialog)
        QtCore.QMetaObject.connectSlotsByName(InstallDialog)

    def retranslateUi(self, InstallDialog):
        _translate = QtCore.QCoreApplication.translate
        self.install_dir_label.setText(_translate("InstallDialog", "Install directory"))
        self.download_only_label.setText(_translate("InstallDialog", "Download only"))
        self.sdl_list_label.setText(_translate("InstallDialog", "Optional packs"))
        self.install_size_info_label.setText(_translate("InstallDialog", "Click verify..."))
        self.install_dialog_label.setText(_translate("InstallDialog", "error"))
        self.install_size_label.setText(_translate("InstallDialog", "Total install size"))
        self.download_only_info_label.setText(_translate("InstallDialog", "Do not try to install."))
        self.max_workers_info_label.setText(_translate("InstallDialog", "Less is slower. (0: Default)"))
        self.warn_message.setText(_translate("InstallDialog", "TextLabel"))
        self.ignore_space_label.setText(_translate("InstallDialog", "Ignore free space"))
        self.download_size_label.setText(_translate("InstallDialog", "Download size"))
        self.force_download_label.setText(_translate("InstallDialog", "Force redownload"))
        self.ignore_space_info_label.setText(_translate("InstallDialog", "Use with caution!"))
        self.max_workers_label.setText(_translate("InstallDialog", "Max workers"))
        self.download_size_info_label.setText(_translate("InstallDialog", "Click verify..."))
        self.cancel_button.setText(_translate("InstallDialog", "Cancel"))
        self.verify_button.setText(_translate("InstallDialog", "Verify"))
        self.install_button.setText(_translate("InstallDialog", "Install"))
        self.warn_label.setText(_translate("InstallDialog", "Warnings"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    InstallDialog = QtWidgets.QDialog()
    ui = Ui_InstallDialog()
    ui.setupUi(InstallDialog)
    InstallDialog.show()
    sys.exit(app.exec_())



import datetime
from logging import getLogger

from PyQt5.QtCore import QThread, pyqtSignal, QSettings
from PyQt5.QtWidgets import QWidget, QMessageBox, QVBoxLayout, QLabel, QGridLayout, QProgressBar, QPushButton, \
    QHBoxLayout, QGroupBox

from custom_legendary.core import LegendaryCore
from custom_legendary.models.downloading import UIUpdate
from custom_legendary.models.game import Game, InstalledGame
from rare.components.dialogs.install_dialog import InstallDialog
from rare.components.tabs.downloads.dl_queue_widget import DlQueueWidget
from rare.components.tabs.downloads.download_thread import DownloadThread
from rare.utils.models import InstallOptionsModel, InstallQueueItemModel
from rare.utils.utils import get_size

logger = getLogger("Download")


class DownloadTab(QWidget):
    finished = pyqtSignal(tuple)
    thread: QThread
    dl_queue = []

    def __init__(self, core: LegendaryCore, updates: list, parent):
        super(DownloadTab, self).__init__(parent=parent)
        self.core = core
        self.layout = QVBoxLayout()
        self.active_game: Game = None
        self.analysis = None

        self.info_layout = QGridLayout()

        self.installing_game = QLabel(self.tr("No active Download"))
        self.info_layout.addWidget(self.installing_game, 0, 0)
        self.dl_speed = QLabel()
        self.info_layout.addWidget(self.dl_speed, 0, 1)
        self.cache_used = QLabel()
        self.info_layout.addWidget(self.cache_used, 1, 0)
        self.downloaded = QLabel()
        self.info_layout.addWidget(self.downloaded, 1, 1)
        self.time_left = QLabel()
        self.info_layout.addWidget(self.time_left, 2, 0)

        self.layout.addLayout(self.info_layout)

        self.mini_layout = QHBoxLayout()
        self.prog_bar = QProgressBar()
        self.mini_layout.addWidget(self.prog_bar)
        self.prog_bar.setMaximum(100)

        self.kill_button = QPushButton(self.tr("Stop Download"))
        self.mini_layout.addWidget(self.kill_button)
        self.kill_button.setDisabled(True)
        self.kill_button.clicked.connect(self.stop_download)

        self.layout.addLayout(self.mini_layout)

        self.queue_widget = DlQueueWidget()
        self.layout.addWidget(self.queue_widget)
        self.queue_widget.update_list.connect(self.update_dl_queue)

        self.updates = QGroupBox(self.tr("Updates"))
        self.layout.addWidget(self.updates)
        self.update_layout = QVBoxLayout()
        self.updates.setLayout(self.update_layout)

        self.updates.setObjectName("group")

        self.update_widgets = {}

        self.update_text = QLabel(self.tr("No updates available"))
        self.update_layout.addWidget(self.update_text)
        self.update_text.setVisible(len(updates) == 0)

        for igame in updates:
            self.add_update(igame)

        self.layout.addStretch(1)

        self.setLayout(self.layout)

    def add_update(self, igame: InstalledGame):
        widget = UpdateWidget(self.core, igame, self)
        self.update_layout.addWidget(widget)
        self.update_widgets[igame.app_name] = widget
        widget.update_signal.connect(self.update_game)
        if QSettings().value("auto_update", False, bool):
            self.update_game(igame.app_name, True)
            widget.update_button.setDisabled(True)

    def update_dl_queue(self, dl_queue):
        self.dl_queue = dl_queue

    def stop_download(self):
        self.thread.kill()

    def install_game(self, queue_item: InstallQueueItemModel):
        if self.active_game is None:
            self.start_installation(queue_item)
        else:
            self.dl_queue.append(queue_item)
            self.queue_widget.update_queue(self.dl_queue)

    def start_installation(self, queue_item: InstallQueueItemModel):
        if self.dl_queue:
            self.dl_queue.pop(0)
            self.queue_widget.update_queue(self.dl_queue)
        self.active_game = queue_item.download.game
        self.thread = DownloadThread(self.core, queue_item)
        self.thread.status.connect(self.status)
        self.thread.statistics.connect(self.statistics)
        self.thread.start()
        self.kill_button.setDisabled(False)
        self.analysis = queue_item.download.analysis
        self.installing_game.setText(self.tr("Installing Game: ") + self.active_game.app_title)

    def status(self, text):
        if text == "dl_finished":
            pass
        elif text == "finish":
            self.installing_game.setText(self.tr("Download finished. Reload library"))
            if QSettings().value("notification", True, bool):
                try:
                    from notifypy import Notify
                except ModuleNotFoundError:
                    logger.warning("No Notification Module found")
                else:
                    notification = Notify()
                    notification.title = self.tr("Installation finished")
                    notification.message = self.tr("Finished Download of game {}").format(self.active_game.app_title)
                    notification.send()
            # QMessageBox.information(self, "Info", "Download finished")
            logger.info("Download finished: " + self.active_game.app_title)

            app_name = self.active_game.app_name
            self.active_game = None

            if self.dl_queue:
                if self.dl_queue[0].download.game.app_name == app_name:
                    self.dl_queue.pop(0)
                    self.queue_widget.update_queue(self.dl_queue)

            if app_name in self.update_widgets.keys():
                self.update_widgets[app_name].setVisible(False)
                self.update_widgets.pop(app_name)
                if len(self.update_widgets) == 0:
                    self.update_text.setVisible(True)

            self.finished.emit((True, app_name))

            self.reset_infos()

            if len(self.dl_queue) != 0:
                self.start_installation(self.dl_queue[0])
            else:
                self.queue_widget.update_queue(self.dl_queue)

        elif text[:5] == "error":
            QMessageBox.warning(self, "warn", "Download error: " + text[6:])

        elif text == "stop":
            self.reset_infos()
            self.active_game = None
            self.finished.emit((False, ""))
            if self.dl_queue:
                self.start_installation(self.dl_queue[0])

    def reset_infos(self):
        self.kill_button.setDisabled(True)
        self.installing_game.setText(self.tr("Installing Game: No active download"))
        self.prog_bar.setValue(0)
        self.dl_speed.setText("")
        self.time_left.setText("")
        self.cache_used.setText("")
        self.downloaded.setText("")
        self.analysis = None

    def statistics(self, ui_update: UIUpdate):
        self.prog_bar.setValue(ui_update.progress)
        self.dl_speed.setText(self.tr("Download speed") + f": {get_size(ui_update.download_speed)}/s")
        self.cache_used.setText(
            self.tr("Cache used") + f": {get_size(ui_update.cache_usage) if ui_update.cache_usage > 1023 else '0KB'}")
        self.downloaded.setText(
            self.tr("Downloaded") + f": {get_size(ui_update.total_downloaded)} / {get_size(self.analysis.dl_size)}")
        self.time_left.setText(self.tr("Time left: ") + self.get_time(ui_update.estimated_time_left))

    def get_time(self, seconds: int) -> str:
        return str(datetime.timedelta(seconds=seconds))

    def update_game(self, app_name: str, auto=False):
        logger.info("Update " + app_name)
        install_dialog = InstallDialog(self.core,
                                       InstallQueueItemModel(options=InstallOptionsModel(app_name=app_name)),
                                       update=True, silent=auto, parent=self)
        install_dialog.result_ready.connect(self.on_install_dialog_closed)
        install_dialog.execute()

    def on_install_dialog_closed(self, download_item: InstallQueueItemModel):
        if download_item:
            self.install_game(download_item)
        else:
            self.update_widgets[download_item.options.app_name].update_button.setDisabled(False)
            self.update_widgets[download_item.options.app_name].update_with_settings.setDisabled(False)


class UpdateWidget(QWidget):
    update_signal = pyqtSignal(str, bool)

    def __init__(self, core: LegendaryCore, game: InstalledGame, parent):
        super(UpdateWidget, self).__init__(parent=parent)
        self.core = core
        self.game = game

        self.layout = QVBoxLayout()
        self.title = QLabel(self.game.title)
        self.layout.addWidget(self.title)

        self.update_button = QPushButton(self.tr("Update Game"))
        self.update_button.clicked.connect(lambda: self.update_game(True))
        self.update_with_settings = QPushButton("Update with settings")
        self.update_with_settings.clicked.connect(lambda: self.update_game(False))
        self.layout.addWidget(self.update_button)
        self.layout.addWidget(self.update_with_settings)
        self.layout.addWidget(QLabel(
            self.tr("Version: ") + self.game.version + " -> " + self.core.get_asset(self.game.app_name,
                                                                                    True).build_version))

        self.setLayout(self.layout)

    def update_game(self, auto: bool):
        self.update_button.setDisabled(True)
        self.update_with_settings.setDisabled(True)
        self.update_signal.emit(self.game.app_name, auto)  # True if settings



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'launch_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtWidgets


class Ui_LaunchDialog(object):
    def setupUi(self, LaunchDialog):
        LaunchDialog.setObjectName("LaunchDialog")
        LaunchDialog.resize(400, 168)
        self.verticalLayout = QtWidgets.QVBoxLayout(LaunchDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.title_label = QtWidgets.QLabel(LaunchDialog)
        self.title_label.setObjectName("title_label")
        self.verticalLayout.addWidget(self.title_label)
        self.image_prog_bar = QtWidgets.QProgressBar(LaunchDialog)
        self.image_prog_bar.setProperty("value", 0)
        self.image_prog_bar.setObjectName("image_prog_bar")
        self.verticalLayout.addWidget(self.image_prog_bar)
        self.image_info = QtWidgets.QLabel(LaunchDialog)
        self.image_info.setObjectName("image_info")
        self.verticalLayout.addWidget(self.image_info)

        self.retranslateUi(LaunchDialog)
        QtCore.QMetaObject.connectSlotsByName(LaunchDialog)

    def retranslateUi(self, LaunchDialog):
        _translate = QtCore.QCoreApplication.translate
        LaunchDialog.setWindowTitle(_translate("LaunchDialog", "Launching Rare"))
        self.title_label.setText(_translate("LaunchDialog", "<h2>Launching Rare</h2>"))
        self.image_info.setText(_translate("LaunchDialog", "Downloading images"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    LaunchDialog = QtWidgets.QDialog()
    ui = Ui_LaunchDialog()
    ui.setupUi(LaunchDialog)
    LaunchDialog.show()
    sys.exit(app.exec_())



import os

from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QSystemTrayIcon, QMenu, QAction

from rare import resources_path


class TrayIcon(QSystemTrayIcon):
    def __init__(self, parent):
        super(TrayIcon, self).__init__(parent)
        self.setIcon(QIcon(os.path.join(resources_path, "images", "Rare.png")))
        self.setVisible(True)
        self.setToolTip("Rare")

        self.menu = QMenu()

        self.start_rare = QAction("Rare")
        self.menu.addAction(self.start_rare)

        self.exit_action = QAction(self.tr("Exit"))
        self.menu.addSeparator()
        self.menu.addAction(self.exit_action)

        self.setContextMenu(self.menu)



import os
import platform
from logging import getLogger

from PyQt5.QtCore import pyqtSignal, QProcess, QSettings, Qt, QByteArray
from PyQt5.QtWidgets import QGroupBox, QMessageBox, QAction

from custom_legendary.core import LegendaryCore
from custom_legendary.models.game import InstalledGame
from rare.components.dialogs.uninstall_dialog import UninstallDialog
from rare.components.extra.Console import ConsoleWindow
from rare.utils import legendary_utils
from rare.utils.utils import create_desktop_link

logger = getLogger("Game")


class BaseInstalledWidget(QGroupBox):
    launch_signal = pyqtSignal(str)
    show_info = pyqtSignal(str)
    finish_signal = pyqtSignal(str)
    update_list = pyqtSignal()
    proc: QProcess()

    def __init__(self, igame: InstalledGame, core: LegendaryCore, pixmap, offline):
        super(BaseInstalledWidget, self).__init__()
        self.igame = igame
        self.core = core
        self.game = self.core.get_game(self.igame.app_name)
        self.pixmap = pixmap
        self.game_running = False
        self.offline = offline
        self.update_available = self.core.get_asset(self.game.app_name, True).build_version != igame.version
        self.data = QByteArray()
        self.setContentsMargins(0, 0, 0, 0)
        self.settings = QSettings()

        self.setContextMenuPolicy(Qt.ActionsContextMenu)
        launch = QAction(self.tr("Launch"), self)
        launch.triggered.connect(self.launch)
        self.addAction(launch)

        if os.path.exists(os.path.expanduser(f"~/Desktop/{self.igame.title}.desktop")) \
                or os.path.exists(os.path.expanduser(f"~/Desktop/{self.igame.title}.lnk")):
            self.create_desktop = QAction(self.tr("Remove Desktop link"))
        else:
            self.create_desktop = QAction(self.tr("Create Desktop link"))

        self.create_desktop.triggered.connect(lambda: self.create_desktop_link("desktop"))
        self.addAction(self.create_desktop)

        if platform.system() == "Linux":
            start_menu_file = os.path.expanduser(f"~/.local/share/applications/{self.igame.title}.desktop")
        elif platform.system() == "Windows":
            start_menu_file = os.path.expandvars("%appdata%/Microsoft/Windows/Start Menu")
        else:
            start_menu_file = ""
        if os.path.exists(start_menu_file):
            self.create_start_menu = QAction(self.tr("Remove start menu link"))
        else:
            self.create_start_menu = QAction(self.tr("Create start menu link"))

        self.create_start_menu.triggered.connect(lambda: self.create_desktop_link("start_menu"))
        self.addAction(self.create_start_menu)

        uninstall = QAction(self.tr("Uninstall"), self)
        uninstall.triggered.connect(self.uninstall)
        self.addAction(uninstall)

    def create_desktop_link(self, type_of_link):
        if platform.system() not in ["Windows", "Linux"]:
            QMessageBox.warning(self, "Warning",
                                f"Create a Desktop link is currently not supported on {platform.system()}")
            return
        if type_of_link == "desktop":
            path = os.path.expanduser(f"~/Desktop/")
        elif type_of_link == "start_menu":
            path = os.path.expanduser("~/.local/share/applications/")
        else:
            return
        if not (os.path.exists(os.path.expanduser(f"{path}{self.igame.title}.desktop"))
                or os.path.exists(os.path.expanduser(f"{path}{self.igame.title}.lnk"))):
            if not create_desktop_link(self.igame.app_name, self.core, type_of_link):
                return
            if type_of_link == "desktop":
                self.create_desktop.setText(self.tr("Remove Desktop link"))
            elif type_of_link == "start_menu":
                self.create_start_menu.setText(self.tr("Remove Start menu link"))
        else:
            if os.path.exists(os.path.expanduser(f"{path}{self.igame.title}.desktop")):
                os.remove(os.path.expanduser(f"{path}{self.igame.title}.desktop"))
            elif os.path.exists(os.path.expanduser(f"{path}{self.igame.title}.lnk")):
                os.remove(os.path.expanduser(f"{path}{self.igame.title}.lnk"))

            if type_of_link == "desktop":
                self.create_desktop.setText(self.tr("Create Desktop link"))
            elif type_of_link == "start_menu":
                self.create_start_menu.setText(self.tr("Create Start menu link"))

    def launch(self, offline=False, skip_version_check=False):
        if QSettings().value("confirm_start", False, bool):
            if not QMessageBox.question(self, "Launch", self.tr("Do you want to launch {}").format(self.game.app_title),
                                        QMessageBox.Yes | QMessageBox.No) == QMessageBox.Yes:
                logger.info("Cancel Startup")
                return 1
        logger.info("Launching " + self.igame.title)
        if offline or self.offline:
            if not self.igame.can_run_offline:
                QMessageBox.warning(self, "Offline",
                                    self.tr("Game cannot run offline. Please start game in Online mode"))
                return

        try:
            self.proc, params = legendary_utils.launch_game(self.core, self.igame.app_name, offline,
                                                            skip_version_check=skip_version_check)
        except Exception as e:
            logger.error(e)
            QMessageBox.warning(self, "Error",
                                str(e))
            return

        if not self.proc:
            logger.error("Could not start process")
            return 1
        self.proc.finished.connect(self.finished)

        if self.settings.value("show_console", False, bool):
            self.console = ConsoleWindow()
            self.console.show()
            self.proc.readyReadStandardOutput.connect(lambda: self.console.log(
                bytes(self.proc.readAllStandardOutput()).decode("utf-8", errors="ignore")))
            self.proc.readyReadStandardError.connect(lambda: self.console.error(
                bytes(self.proc.readAllStandardOutput()).decode("utf-8", errors="ignore")))

        else:
            self.proc.readyReadStandardOutput.connect(self.stdout)
            self.proc.readyReadStandardError.connect(self.stderr)

        self.proc.start(params[0], params[1:])
        self.launch_signal.emit(self.igame.app_name)
        self.game_running = True

        return 0

    def stdout(self):
        data = self.proc.readAllStandardOutput()
        stdout = bytes(data).decode("utf-8", errors="ignore")
        print(stdout)

    def stderr(self):
        stderr = bytes(self.proc.readAllStandardError()).decode("utf-8", errors="ignore")
        print(stderr)
        # QMessageBox.warning(self, "Warning", stderr + "\nSee ~/.cache/rare/logs/")

    def finished(self, exit_code):
        logger.info("Game exited with exit code: " + str(exit_code))
        self.finish_signal.emit(self.game.app_name)
        self.game_running = False
        if self.settings.value("show_console", False, bool):
            self.console.log(f"Game exited with code: {exit_code}")

    def uninstall(self):
        infos = UninstallDialog(self.game).get_information()
        if infos == 0:
            print("Cancel Uninstall")
            return
        legendary_utils.uninstall(self.game.app_name, self.core, infos)
        self.update_list.emit(self.game.app_name)



from logging import getLogger

import psutil
from PyQt5.QtCore import Qt, pyqtSignal, QSettings, QTimer
from PyQt5.QtWidgets import QScrollArea, QWidget, QLabel, QVBoxLayout, QStackedWidget

from custom_legendary.core import LegendaryCore
from rare.components.tabs.games.game_widgets.base_installed_widget import BaseInstalledWidget
from rare.components.tabs.games.game_widgets.installed_icon_widget import GameWidgetInstalled
from rare.components.tabs.games.game_widgets.installed_list_widget import InstalledListWidget
from rare.components.tabs.games.game_widgets.uninstalled_icon_widget import IconWidgetUninstalled
from rare.components.tabs.games.game_widgets.uninstalled_list_widget import ListWidgetUninstalled
from rare.utils.extra_widgets import FlowLayout
from rare.utils.models import InstallOptionsModel
from rare.utils.utils import download_image, get_uninstalled_pixmap, get_pixmap

logger = getLogger("Game list")


class GameList(QStackedWidget):
    install_game = pyqtSignal(InstallOptionsModel)
    show_game_info = pyqtSignal(str)
    update_game = pyqtSignal()
    game_exited = pyqtSignal(str)
    game_started = pyqtSignal(str)
    show_uninstalled_info = pyqtSignal(str)

    running_games = []
    active_game = ("", 0)

    def __init__(self, core: LegendaryCore, parent, offline):
        super(GameList, self).__init__(parent=parent)
        self.core = core
        self.setObjectName("list_widget")
        self.offline = offline

        self.settings = QSettings()
        icon_view = self.settings.value("icon_view", True, bool)
        self.procs = []
        for proc in psutil.process_iter():
            try:
                self.procs.append((proc.name(), proc.pid))
            except psutil.ZombieProcess:
                continue
            except psutil.NoSuchProcess:
                continue
            except Exception:
                continue
        self.init_ui(icon_view)

    def init_ui(self, icon_view=True):
        self.icon_scrollarea = QScrollArea(parent=self)
        self.icon_widget = QWidget(parent=self.icon_scrollarea)
        self.list_scrollarea = QScrollArea(parent=self)
        self.list_widget = QWidget(parent=self.list_scrollarea)

        self.icon_scrollarea.setWidgetResizable(True)
        self.icon_scrollarea.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)

        self.list_scrollarea.setWidgetResizable(True)
        self.list_scrollarea.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)

        self.info_text = self.tr("Installed Games: {}    Available Games: {}").format(
            len(self.core.get_installed_list()),
            len(self.core.get_game_list(update_assets=True)))

        self.icon_parent_layout = QVBoxLayout()
        self.icon_parent_layout.addWidget(QLabel(self.info_text))

        self.icon_layout = FlowLayout()
        self.list_layout = QVBoxLayout()
        self.list_layout.addWidget(QLabel(self.info_text))

        self.updates = []
        self.widgets = {}
        if not self.offline:
            self.bit32 = [i.app_name for i in self.core.get_game_and_dlc_list(True, "Win32")[0]]
            self.mac_games = [i.app_name for i in self.core.get_game_and_dlc_list(True, "Mac")[0]]
        else:
            self.bit32 = []
            self.mac_games = []
        self.installed = sorted(self.core.get_installed_list(), key=lambda x: x.title)

        # Installed Games
        for igame in self.installed:
            icon_widget, list_widget = self.add_installed_widget(igame)
            self.icon_layout.addWidget(icon_widget)
            self.list_layout.addWidget(list_widget)
        if not self.offline:
            self.uninstalled_games = []
            installed = [i.app_name for i in self.core.get_installed_list()]
            # get Uninstalled games
            games, self.dlcs = self.core.get_game_and_dlc_list(update_assets=not self.offline)
            for game in sorted(games, key=lambda x: x.app_title):
                if not game.app_name in installed:
                    self.uninstalled_games.append(game)

            # add uninstalled games

            for game in self.uninstalled_games:
                icon_widget, list_widget = self.add_uninstalled_widget(game)

                self.icon_layout.addWidget(icon_widget)
                self.list_layout.addWidget(list_widget)
        else:
            self.dlcs = []

        self.icon_parent_layout.addLayout(self.icon_layout)
        self.icon_parent_layout.addStretch(1)
        self.list_layout.addStretch(1)
        self.icon_widget.setLayout(self.icon_parent_layout)
        self.list_widget.setLayout(self.list_layout)

        self.icon_scrollarea.setWidget(self.icon_widget)
        self.list_scrollarea.setWidget(self.list_widget)

        self.addWidget(self.icon_scrollarea)
        self.addWidget(self.list_scrollarea)

        if not icon_view:
            self.setCurrentIndex(1)

        if filter_games := self.settings.value("filter", "", str):
            self.filter(filter_games)

    def add_uninstalled_widget(self, game):
        pixmap = get_uninstalled_pixmap(game.app_name)
        if pixmap.isNull():
            logger.info(game.app_title + " has a corrupt image. Reloading...")
            download_image(game, force=True)
            pixmap = get_uninstalled_pixmap(game.app_name)

        icon_widget = IconWidgetUninstalled(game, self.core, pixmap)
        icon_widget.show_uninstalled_info.connect(self.show_install_info)

        list_widget = ListWidgetUninstalled(self.core, game, pixmap)
        list_widget.show_uninstalled_info.connect(self.show_install_info)

        self.widgets[game.app_name] = (icon_widget, list_widget)

        return icon_widget, list_widget

    def add_installed_widget(self, igame):
        pixmap = get_pixmap(igame.app_name)

        if pixmap.isNull():
            logger.info(igame.title + " has a corrupt image.")
            download_image(self.core.get_game(igame.app_name), force=True)
            pixmap = get_pixmap(igame.app_name)

        icon_widget = GameWidgetInstalled(igame, self.core, pixmap, self.offline)
        # self.icon_layout.addWidget(icon_widget)

        list_widget = InstalledListWidget(igame, self.core, pixmap, self.offline)
        # self.list_layout.addWidget(list_widget)

        self.widgets[igame.app_name] = (icon_widget, list_widget)

        icon_widget.show_info.connect(self.show_game_info.emit)
        list_widget.show_info.connect(self.show_game_info.emit)

        icon_widget.launch_signal.connect(self.launch)
        icon_widget.finish_signal.connect(self.finished)
        icon_widget.update_list.connect(self.update_list)
        list_widget.launch_signal.connect(self.launch)
        list_widget.finish_signal.connect(self.finished)
        list_widget.update_list.connect(self.update_list)

        if icon_widget.update_available:
            self.updates.append(igame)

        active, pid = self.check_for_active_game(igame)
        if active:
            # Only one game works: Workaround for Satisfactory EA and Exp.
            self.launch(igame.app_name)
            icon_widget.game_running = True
            list_widget.game_running = True
            self.active_game = (igame.app_name, pid)
            self.timer = QTimer()
            self.timer.timeout.connect(self.is_finished)
            self.timer.start(10000)

        return icon_widget, list_widget

    def is_finished(self):
        if psutil.pid_exists(self.active_game[1]):
            self.timer.start(10000)
        else:
            self.finished(self.active_game[0])
            del self.timer

    def check_for_active_game(self, igame):
        executable = igame.executable.split("/")[-1].split("\\")[-1]
        if executable.endswith(".exe"):
            executable = executable[:-4]  # remove .exe

        for i, pid in self.procs:
            if executable in i:
                # Workaround for Satisfactory: Check Cmdline args
                if igame.app_name in ["CrabEA", "CrabTest"]:
                    p = psutil.Process(pid)
                    if not igame.install_path.split("/")[-1].split("\\")[-1] in " ".join(p.cmdline()):
                        return False, 0
                return True, pid
        return False, 0

    def show_install_info(self, app_name):
        self.show_uninstalled_info.emit(app_name)

    def finished(self, app_name):
        self.running_games.remove(app_name)
        self.active_game = ("", 0)
        self.widgets[app_name][0].info_text = ""
        self.widgets[app_name][0].info_label.setText("")
        self.widgets[app_name][1].launch_button.setDisabled(False)
        self.widgets[app_name][1].launch_button.setText(self.tr("Launch"))
        if self.widgets[app_name][0].game.supports_cloud_saves:
            if not self.settings.value(f"{app_name}/auto_sync_cloud", True, bool) \
                    and not self.settings.value("auto_sync_cloud", True, bool):
                logger.info("Auto saves disabled")
                return

            self.widgets[app_name][0].info_text = self.tr("Sync CLoud saves")
            self.widgets[app_name][0].info_label.setText(self.tr("Sync CLoud saves"))
            self.widgets[app_name][1].info_label.setText(self.tr("Sync CLoud saves"))
        self.game_exited.emit(app_name)

    def launch(self, app_name):
        self.running_games.append(app_name)
        self.game_started.emit(app_name)
        self.widgets[app_name][0].info_text = self.tr("Game running")
        self.widgets[app_name][0].info_label.setText(self.tr("Game running"))
        self.widgets[app_name][1].launch_button.setDisabled(True)
        self.widgets[app_name][1].launch_button.setText(self.tr("Game running"))

    def search(self, text: str):
        for t in self.widgets.values():
            for w in t:
                if text.lower() in w.game.app_title.lower() + w.game.app_name.lower():
                    w.setVisible(True)
                else:
                    w.setVisible(False)

    def filter(self, filter="installed"):
        for t in self.widgets.values():
            for w in t:
                if filter == "installed":
                    w.setVisible(self.core.is_installed(w.game.app_name))
                elif filter == "offline":
                    if self.core.is_installed(w.game.app_name):
                        w.setVisible(w.igame.can_run_offline)
                    else:
                        w.setVisible(False)
                elif filter == "32bit" and self.bit32:
                    w.setVisible(w.game.app_name in self.bit32)
                elif filter == "mac":
                    w.setVisible(w.game.app_name in self.mac_games)
                else:
                    # All visible
                    w.setVisible(True)
        self.settings.setValue("filter", filter)

    def update_list(self, app_name=None):
        # self.settings.setValue("icon_view", icon_view)
        if app_name:
            if widgets := self.widgets.get(app_name):

                # from update
                if self.core.is_installed(widgets[0].game.app_name) and isinstance(widgets[0], BaseInstalledWidget):
                    igame = self.core.get_installed_game(app_name)
                    for w in widgets:
                        w.igame = igame
                        w.update_available = self.core.get_asset(w.game.app_name, True).build_version != igame.version
                    widgets[0].info_label.setText("")
                    widgets[0].info_text = ""
                # new installed
                elif self.core.is_installed(widgets[0].game.app_name) and not isinstance(widgets[0],
                                                                                         BaseInstalledWidget):
                    self.widgets.pop(widgets[0].game.app_name)

                    # QWidget().setLayout(self.icon_layout)

                    igame = self.core.get_installed_game(app_name)
                    self.add_installed_widget(igame)

                    self._update_games()

                # uninstalled
                elif not self.core.is_installed(widgets[0].game.app_name) and isinstance(widgets[0],
                                                                                         BaseInstalledWidget):
                    self.list_layout.removeWidget(widgets[1])
                    self.icon_layout.removeWidget(widgets[0])

                    self.widgets.pop(app_name)

                    game = self.core.get_game(app_name, True)
                    self.add_uninstalled_widget(game)

                    self._update_games()

        else:
            installed_names = [i.app_name for i in self.core.get_installed_list()]
            # get Uninstalled games
            uninstalled_names = []
            games = self.core.get_game_list(True)
            for game in sorted(games, key=lambda x: x.app_title):
                if not game.app_name in installed_names:
                    uninstalled_names.append(game.app_name)

            new_installed_games = list(set(installed_names) - set([i.app_name for i in self.installed]))
            new_uninstalled_games = list(set(uninstalled_names) - set([i.app_name for i in self.uninstalled_games]))

            if (not new_uninstalled_games) and (not new_installed_games):
                return

            if new_installed_games:
                for name in new_installed_games:
                    self.icon_layout.removeWidget(self.widgets[app_name][0])
                    self.list_layout.removeWidget(self.widgets[app_name][1])

                    self.widgets.pop(name)

                    igame = self.core.get_installed_game(name)
                    self.add_installed_widget(igame)

                for name in new_uninstalled_games:
                    self.icon_layout.removeWidget(self.widgets[app_name][0])
                    self.list_layout.removeWidget(self.widgets[app_name][1])

                    self.widgets.pop(name)

                    game = self.core.get_game(name, True)
                    self.add_uninstalled_widget(game)

                for igame in sorted(self.core.get_installed_list(), key=lambda x: x.title):
                    i_widget, list_widget = self.widgets[igame.app_name]

                    self.icon_layout.addWidget(i_widget)
                    self.list_layout.addWidget(list_widget)

                # get Uninstalled games
                games, self.dlcs = self.core.get_game_and_dlc_list()
                for game in sorted(games, key=lambda x: x.app_title):
                    if not game.app_name in installed_names:
                        self.uninstalled_names.append(game)
                for name in uninstalled_names:
                    i_widget, list_widget = self.widgets[name]
                    self.icon_layout.addWidget(i_widget)
                    self.list_layout.addWidget(list_widget)

    def _update_games(self):
        # new layouts to remove from old layout
        icon_layout = FlowLayout()
        # QWidget().setLayout(self.list_layout)
        list_layout = QVBoxLayout()

        for igame in sorted(self.core.get_installed_list(), key=lambda x: x.title):
            i_widget, l_widget = self.widgets[igame.app_name]
            icon_layout.addWidget(i_widget)
            list_layout.addWidget(l_widget)

        self.uninstalled_names = []
        installed_names = [i.app_name for i in self.core.get_installed_list()]
        # get Uninstalled games
        games, self.dlcs = self.core.get_game_and_dlc_list()
        for game in sorted(games, key=lambda x: x.app_title):
            if not game.app_name in installed_names:
                self.uninstalled_names.append(game)
        for game in self.uninstalled_names:
            i_widget, list_widget = self.widgets[game.app_name]
            icon_layout.addWidget(i_widget)
            list_layout.addWidget(list_widget)

        QWidget().setLayout(self.icon_layout)
        QWidget().setLayout(self.list_layout)

        self.icon_widget = QWidget()
        self.list_widget = QWidget()

        self.icon_widget.setLayout(icon_layout)
        self.list_widget.setLayout(list_layout)

        self.list_scrollarea.setWidget(QWidget())
        self.icon_scrollarea.setWidget(QWidget())

        self.icon_scrollarea.setWidget(self.icon_widget)
        self.list_scrollarea.setWidget(self.list_widget)

        self.icon_layout = icon_layout
        self.list_layout = list_layout

        self.icon_widget.setLayout(self.icon_layout)
        self.list_widget.setLayout(self.list_layout)

        self.update()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'about.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_About(object):
    def setupUi(self, About):
        About.setObjectName("About")
        About.resize(499, 396)
        self.gridLayout = QtWidgets.QGridLayout(About)
        self.gridLayout.setObjectName("gridLayout")
        self.open_browser = QtWidgets.QPushButton(About)
        self.open_browser.setObjectName("open_browser")
        self.gridLayout.addWidget(self.open_browser, 4, 1, 1, 1)
        self.version_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.version_label.setFont(font)
        self.version_label.setObjectName("version_label")
        self.gridLayout.addWidget(self.version_label, 2, 0, 1, 1, QtCore.Qt.AlignRight)
        self.license_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.license_label.setFont(font)
        self.license_label.setObjectName("license_label")
        self.gridLayout.addWidget(self.license_label, 7, 0, 1, 1, QtCore.Qt.AlignRight)
        self.lgd_dev_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lgd_dev_label.setFont(font)
        self.lgd_dev_label.setObjectName("lgd_dev_label")
        self.gridLayout.addWidget(self.lgd_dev_label, 6, 0, 1, 1, QtCore.Qt.AlignRight)
        self.dev = QtWidgets.QLabel(About)
        self.dev.setText("<a href=\'https://github.com/Dummerle\'>Dummerle</a>")
        self.dev.setOpenExternalLinks(True)
        self.dev.setObjectName("dev")
        self.gridLayout.addWidget(self.dev, 5, 1, 1, 1)
        self.update_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.update_label.setFont(font)
        self.update_label.setObjectName("update_label")
        self.gridLayout.addWidget(self.update_label, 3, 0, 1, 1, QtCore.Qt.AlignRight)
        self.version = QtWidgets.QLabel(About)
        self.version.setText("error")
        self.version.setObjectName("version")
        self.gridLayout.addWidget(self.version, 2, 1, 1, 1)
        self.dev_label = QtWidgets.QLabel(About)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.dev_label.setFont(font)
        self.dev_label.setObjectName("dev_label")
        self.gridLayout.addWidget(self.dev_label, 5, 0, 1, 1, QtCore.Qt.AlignRight)
        self.license = QtWidgets.QLabel(About)
        self.license.setObjectName("license")
        self.gridLayout.addWidget(self.license, 7, 1, 1, 1)
        self.update = QtWidgets.QLabel(About)
        self.update.setText("error")
        self.update.setObjectName("update")
        self.gridLayout.addWidget(self.update, 3, 1, 1, 1)
        self.lgd_dev = QtWidgets.QLabel(About)
        self.lgd_dev.setText("<a href=\'https://github.com/derrod/\'>derrod</a>")
        self.lgd_dev.setOpenExternalLinks(True)
        self.lgd_dev.setObjectName("lgd_dev")
        self.gridLayout.addWidget(self.lgd_dev, 6, 1, 1, 1)
        self.info_text = QtWidgets.QLabel(About)
        self.info_text.setWordWrap(True)
        self.info_text.setOpenExternalLinks(True)
        self.info_text.setObjectName("info_text")
        self.gridLayout.addWidget(self.info_text, 14, 0, 1, 3)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 15, 0, 1, 3)
        self.title = QtWidgets.QLabel(About)
        self.title.setObjectName("title")
        self.gridLayout.addWidget(self.title, 1, 0, 1, 3)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 2, 2, 6, 1)

        self.retranslateUi(About)
        QtCore.QMetaObject.connectSlotsByName(About)

    def retranslateUi(self, About):
        _translate = QtCore.QCoreApplication.translate
        About.setWindowTitle(_translate("About", "About"))
        self.open_browser.setText(_translate("About", "Download latest release"))
        self.version_label.setText(_translate("About", "Version"))
        self.license_label.setText(_translate("About", "License"))
        self.lgd_dev_label.setText(_translate("About", "Legendary Developer"))
        self.dev.setToolTip(_translate("About", "Github"))
        self.update_label.setText(_translate("About", "Update available"))
        self.dev_label.setText(_translate("About", "Rare Developer"))
        self.license.setText(_translate("About", "GNU General Public License v3.0"))
        self.lgd_dev.setToolTip(_translate("About", "Github"))
        self.info_text.setText(_translate("About",
                                          "This is a beta version, so you can get bugs. If you get a bug, please report it by creating an Issue on <a href=\'https://github.com/Dummerle/Rare/issues\'>Github</a>. You can also contact me on Discord (Dummerle#7419). Or you can join the <a href=\'https://discord.gg/YvmABK9YSk\'>Discord server</a>"))
        self.title.setText(_translate("About", "<h2>About</h2>"))


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    About = QtWidgets.QWidget()
    ui = Ui_About()
    ui.setupUi(About)
    About.show()
    sys.exit(app.exec_())



#!/usr/bin/python

import os
import platform
from argparse import ArgumentParser

from rare import __version__
from rare.utils import singleton, utils


def main():
    parser = ArgumentParser()
    parser.add_argument("-V", "--version", action="store_true", help="Shows version and exits")
    parser.add_argument("-S", "--silent", action="store_true",
                        help="Launch Rare in background. Open it from System Tray Icon")
    parser.add_argument("--offline", action="store_true", help="Launch Rare in offline mode")
    if platform.system() != "Windows":
        parser.add_argument("--disable-protondb", action="store_true", dest="disable_protondb",
                            help="Do not download and check data from ProtonDB. Disable it, if you don't need grades")
        parser.add_argument("--enable-protondb", action="store_true", dest="enable_protondb",
                            help="Enable ProtonDB data, after disabled")

    parser.add_argument("--desktop-shortcut", action="store_true", dest="desktop_shortcut",
                        help="Use this, if there is no link on desktop to start Rare")
    parser.add_argument("--startmenu-shortcut", action="store_true", dest="startmenu_shortcut",
                        help="Use this, if there is no link in start menu to launch Rare")
    subparsers = parser.add_subparsers(title="Commands", dest="subparser")

    launch_parser = subparsers.add_parser("launch")
    launch_parser.add_argument('app_name', help='Name of the app', metavar='<App Name>')

    args = parser.parse_args()

    if args.desktop_shortcut:
        utils.create_rare_desktop_link("desktop")
        print("Link created")
    if args.startmenu_shortcut:
        utils.create_rare_desktop_link("start_menu")
        print("link created")

    if args.version:
        print(__version__)
        exit(0)
    try:
        # this object only allows one instance pre machine
        me = singleton.SingleInstance()
    except singleton.SingleInstanceException:
        print("Rare is already running")

        with open(os.path.expanduser("~/.cache/rare/lockfile"), "w") as file:
            if args.subparser == "launch":
                file.write("launch " + args.app_name)
            else:
                file.write("start")
            file.close()

        exit(0)

    if args.subparser == "launch":
        args.silent = True

    from rare.app import start
    start(args)


if __name__ == '__main__':
    main()



from logging import getLogger

from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QGroupBox, QVBoxLayout, QLabel, QHBoxLayout, QPushButton, QWidget
from qtawesome import icon

from rare.utils.models import InstallQueueItemModel

logger = getLogger("QueueWidget")


class DlWidget(QWidget):
    move_up = pyqtSignal(str)  # app_name
    move_down = pyqtSignal(str)  # app_name
    remove = pyqtSignal(str)  # app_name

    def __init__(self, index, queue_item: InstallQueueItemModel):
        super(DlWidget, self).__init__()
        self.app_name = queue_item.download.game.app_name
        self.layout = QHBoxLayout()

        self.left_layout = QVBoxLayout()
        self.move_up_button = QPushButton(icon("fa.arrow-up", color="white"), "")
        if index == 0:
            self.move_up_button.setDisabled(True)
        self.move_up_button.clicked.connect(lambda: self.move_up.emit(self.app_name))
        self.move_up_button.setFixedWidth(20)
        self.left_layout.addWidget(self.move_up_button)

        self.move_down_buttton = QPushButton(icon("fa.arrow-down", color="white"), "")
        self.move_down_buttton.clicked.connect(lambda: self.move_down.emit(self.app_name))
        self.left_layout.addWidget(self.move_down_buttton)
        self.move_down_buttton.setFixedWidth(20)

        self.layout.addLayout(self.left_layout)

        self.right_layout = QVBoxLayout()
        self.title = QLabel(queue_item.download.game.app_title)
        self.right_layout.addWidget(self.title)

        dl_size = queue_item.download.analysis.dl_size
        install_size = queue_item.download.analysis.install_size

        self.size = QHBoxLayout()

        self.size.addWidget(QLabel(self.tr("Download size: {} GB").format(round(dl_size / 1024 ** 3, 2))))
        self.size.addWidget(QLabel(self.tr("Install size: {} GB").format(round(install_size / 1024 ** 3, 2))))
        self.right_layout.addLayout(self.size)

        self.delete = QPushButton(self.tr("Remove Download"))
        self.delete.clicked.connect(lambda: self.remove.emit(self.app_name))
        self.right_layout.addWidget(self.delete)

        self.layout.addLayout(self.right_layout)
        self.setLayout(self.layout)


class DlQueueWidget(QGroupBox):
    update_list = pyqtSignal(list)
    dl_queue = []

    def __init__(self):

        super(DlQueueWidget, self).__init__()
        self.setTitle(self.tr("Download Queue"))
        self.layout = QVBoxLayout()
        self.setObjectName("group")
        self.text = QLabel(self.tr("No downloads in queue"))
        self.layout.addWidget(self.text)

        self.setLayout(self.layout)

    def update_queue(self, dl_queue: list):
        logger.info("Update Queue " + ", ".join(i.download.game.app_title for i in dl_queue))
        self.dl_queue = dl_queue
        self.setLayout(QVBoxLayout())
        QWidget().setLayout(self.layout)
        self.layout = QVBoxLayout()

        if len(dl_queue) == 0:
            self.layout.addWidget(QLabel(self.tr("No downloads in queue")))
            self.setLayout(self.layout)
            return

        for index, item in enumerate(dl_queue):
            widget = DlWidget(index, item)
            widget.remove.connect(self.remove)
            widget.move_up.connect(self.move_up)
            widget.move_down.connect(self.move_down)
            self.layout.addWidget(widget)
            if index + 1 == len(dl_queue):
                widget.move_down_buttton.setDisabled(True)

        self.setLayout(self.layout)

    def remove(self, app_name):
        for index, i in enumerate(self.dl_queue):
            if i.download.game.app_name == app_name:
                self.dl_queue.pop(index)
                break
        else:
            logger.warning("BUG! ", app_name)
            return
        self.update_list.emit(self.dl_queue)
        self.update_queue(self.dl_queue)

    def move_up(self, app_name):
        index: int

        for i, item in enumerate(self.dl_queue):
            if item.download.game.app_name == app_name:
                index = i
                break
        else:
            logger.warning("Could not find appname" + app_name)
            return
        self.dl_queue.insert(index - 1, self.dl_queue.pop(index))
        self.update_list.emit(self.dl_queue)
        self.update_queue(self.dl_queue)

    def move_down(self, app_name):
        index: int

        for i, item in enumerate(self.dl_queue):
            if item.download.game.app_name == app_name:
                index = i
                break
        else:
            logger.warning("Could not find appname" + app_name)
            return
        self.dl_queue.insert(index + 1, self.dl_queue.pop(index))
        self.update_list.emit(self.dl_queue)
        self.update_queue(self.dl_queue)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'legendary.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_legendary_settings(object):
    def setupUi(self, legendary_settings):
        legendary_settings.setObjectName("legendary_settings")
        legendary_settings.resize(532, 383)
        self.settings = QtWidgets.QWidget()
        self.settings.setObjectName("settings")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.settings)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gb_clean = QtWidgets.QGroupBox(self.settings)
        self.gb_clean.setObjectName("gb_clean")
        self.layout_clean = QtWidgets.QVBoxLayout(self.gb_clean)
        self.layout_clean.setObjectName("layout_clean")
        self.clean_button_without_manifests = QtWidgets.QPushButton(self.gb_clean)
        self.clean_button_without_manifests.setObjectName("clean_button_without_manifests")
        self.layout_clean.addWidget(self.clean_button_without_manifests)
        self.clean_button = QtWidgets.QPushButton(self.gb_clean)
        self.clean_button.setObjectName("clean_button")
        self.layout_clean.addWidget(self.clean_button)
        self.gridLayout_2.addWidget(self.gb_clean, 0, 1, 1, 1)
        self.egl_sync = QtWidgets.QGroupBox(self.settings)
        self.egl_sync.setObjectName("egl_sync")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.egl_sync)
        self.verticalLayout.setObjectName("verticalLayout")
        self.sync_button = QtWidgets.QPushButton(self.egl_sync)
        self.sync_button.setObjectName("sync_button")
        self.verticalLayout.addWidget(self.sync_button)
        self.gridLayout_2.addWidget(self.egl_sync, 1, 1, 1, 1)
        self.gb_downloads = QtWidgets.QGroupBox(self.settings)
        self.gb_downloads.setObjectName("gb_downloads")
        self.layout_downloads = QtWidgets.QGridLayout(self.gb_downloads)
        self.layout_downloads.setObjectName("layout_downloads")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.layout_downloads.addItem(spacerItem, 0, 4, 1, 1)
        self.max_worker_select = QtWidgets.QSpinBox(self.gb_downloads)
        self.max_worker_select.setObjectName("max_worker_select")
        self.layout_downloads.addWidget(self.max_worker_select, 0, 1, 1, 1)
        self.lbl_max_workers = QtWidgets.QLabel(self.gb_downloads)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lbl_max_workers.sizePolicy().hasHeightForWidth())
        self.lbl_max_workers.setSizePolicy(sizePolicy)
        self.lbl_max_workers.setObjectName("lbl_max_workers")
        self.layout_downloads.addWidget(self.lbl_max_workers, 0, 0, 1, 1, QtCore.Qt.AlignRight)
        self.lbl_max_workers_info = QtWidgets.QLabel(self.gb_downloads)
        font = QtGui.QFont()
        font.setItalic(True)
        self.lbl_max_workers_info.setFont(font)
        self.lbl_max_workers_info.setObjectName("lbl_max_workers_info")
        self.layout_downloads.addWidget(self.lbl_max_workers_info, 0, 2, 1, 1)
        self.gridLayout_2.addWidget(self.gb_downloads, 1, 0, 1, 1)
        self.gb_install_dir = QtWidgets.QGroupBox(self.settings)
        self.gb_install_dir.setObjectName("gb_install_dir")
        self.layout_install_dir = QtWidgets.QVBoxLayout(self.gb_install_dir)
        self.layout_install_dir.setObjectName("layout_install_dir")
        self.gridLayout_2.addWidget(self.gb_install_dir, 0, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem1, 2, 1, 1, 1)
        legendary_settings.addWidget(self.settings)
        self.egl_sync_page = QtWidgets.QWidget()
        self.egl_sync_page.setObjectName("egl_sync_page")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.egl_sync_page)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.back_button = QtWidgets.QPushButton(self.egl_sync_page)
        self.back_button.setObjectName("back_button")
        self.verticalLayout_2.addWidget(self.back_button)
        self.title = QtWidgets.QLabel(self.egl_sync_page)
        self.title.setObjectName("title")
        self.verticalLayout_2.addWidget(self.title)
        self.pathedit_placeholder = QtWidgets.QVBoxLayout()
        self.pathedit_placeholder.setObjectName("pathedit_placeholder")
        self.verticalLayout_2.addLayout(self.pathedit_placeholder)
        self.path_info = QtWidgets.QLabel(self.egl_sync_page)
        self.path_info.setObjectName("path_info")
        self.verticalLayout_2.addWidget(self.path_info)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.exportable_games = QtWidgets.QGroupBox(self.egl_sync_page)
        self.exportable_games.setObjectName("exportable_games")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.exportable_games)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.gridLayout.addWidget(self.exportable_games, 0, 0, 1, 1)
        self.importable_games = QtWidgets.QGroupBox(self.egl_sync_page)
        self.importable_games.setObjectName("importable_games")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.importable_games)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.gridLayout.addWidget(self.importable_games, 0, 1, 1, 1)
        self.export_all_button = QtWidgets.QPushButton(self.egl_sync_page)
        self.export_all_button.setObjectName("export_all_button")
        self.gridLayout.addWidget(self.export_all_button, 1, 0, 1, 1)
        self.import_all_button = QtWidgets.QPushButton(self.egl_sync_page)
        self.import_all_button.setObjectName("import_all_button")
        self.gridLayout.addWidget(self.import_all_button, 1, 1, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.groupBox = QtWidgets.QGroupBox(self.egl_sync_page)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.sync_once_button = QtWidgets.QPushButton(self.groupBox)
        self.sync_once_button.setObjectName("sync_once_button")
        self.horizontalLayout.addWidget(self.sync_once_button)
        self.enable_sync_button = QtWidgets.QPushButton(self.groupBox)
        self.enable_sync_button.setObjectName("enable_sync_button")
        self.horizontalLayout.addWidget(self.enable_sync_button)
        self.verticalLayout_2.addWidget(self.groupBox)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem2)
        legendary_settings.addWidget(self.egl_sync_page)

        self.retranslateUi(legendary_settings)
        legendary_settings.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(legendary_settings)

    def retranslateUi(self, legendary_settings):
        _translate = QtCore.QCoreApplication.translate
        legendary_settings.setWindowTitle(_translate("legendary_settings", "StackedWidget"))
        self.gb_clean.setTitle(_translate("legendary_settings", "Cleanup"))
        self.clean_button_without_manifests.setText(_translate("legendary_settings", "Clean, but keep manifests"))
        self.clean_button.setText(_translate("legendary_settings", "Remove everything"))
        self.egl_sync.setTitle(_translate("legendary_settings", "EGL Sync"))
        self.sync_button.setText(_translate("legendary_settings", "Sync Settings"))
        self.gb_downloads.setTitle(_translate("legendary_settings", "Download Settings"))
        self.lbl_max_workers.setText(_translate("legendary_settings", "Max Workers"))
        self.lbl_max_workers_info.setText(_translate("legendary_settings", "Less is slower (0: Default)"))
        self.gb_install_dir.setTitle(_translate("legendary_settings", "Default Installation Directory"))
        self.back_button.setText(_translate("legendary_settings", "Back"))
        self.title.setText(_translate("legendary_settings", "<h2>Sync Games with Epic Games Store</h2>"))
        self.path_info.setText(_translate("legendary_settings", "TextLabel"))
        self.exportable_games.setTitle(_translate("legendary_settings", "Exportable Games"))
        self.importable_games.setTitle(_translate("legendary_settings", "Importable"))
        self.export_all_button.setText(_translate("legendary_settings", "Export all Games"))
        self.import_all_button.setText(_translate("legendary_settings", "Import all Games"))
        self.groupBox.setTitle(_translate("legendary_settings", "Enable Sync"))
        self.sync_once_button.setText(_translate("legendary_settings", "Sync once"))
        self.enable_sync_button.setText(_translate("legendary_settings", "Enable automatic sync"))


if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    legendary_settings = QtWidgets.QStackedWidget()
    ui = Ui_legendary_settings()
    ui.setupUi(legendary_settings)
    legendary_settings.show()
    sys.exit(app.exec_())



import difflib
import json
import os
from datetime import date

import requests
from PyQt5.QtCore import QThread, pyqtSignal

from custom_legendary.core import LegendaryCore
from rare import cache_dir, data_dir

replace_chars = ",;.:-_ "

file = os.path.join(cache_dir, "game_list.json")
url = "https://api.steampowered.com/ISteamApps/GetAppList/v2/"


class SteamWorker(QThread):
    app_name = ""
    rating_signal = pyqtSignal(str)

    def __init__(self, core: LegendaryCore):
        super(SteamWorker, self).__init__()
        self.core = core
        self.ratings = {"platinum": self.tr("Platinum"),
                        "gold": self.tr("Gold"),
                        "silver": self.tr("Silver"),
                        "bronze": self.tr("Bronze"),
                        "fail": self.tr("Could not get grade"),
                        "pending": self.tr("Could not get grade")
                        }

    def set_app_name(self, app_name: str):
        self.app_name = app_name

    def run(self) -> None:
        self.rating_signal.emit(self.ratings[get_rating(self.app_name, self.core)])


def get_rating(app_name: str, core: LegendaryCore):
    if os.path.exists(p := os.path.join(data_dir, "steam_ids.json")):
        grades = json.load(open(p))
    else:
        grades = {}

    if not grades.get(app_name):
        steam_id = get_steam_id(core.get_game(app_name).app_title)
        grade = get_grade(steam_id)
        grades[app_name] = {
            "steam_id": steam_id,
            "grade": grade
        }
        with open(os.path.join(data_dir, "steam_ids.json"), "w") as f:
            f.write(json.dumps(grades))
            f.close()
        return grade
    else:
        return grades[app_name].get("grade")


# you should iniciate the module with the game's steam code
def get_grade(steam_code):
    if steam_code == 0:
        return "fail"
    steam_code = str(steam_code)
    url = 'https://www.protondb.com/api/v1/reports/summaries/'
    res = requests.get(url + steam_code + '.json')
    try:
        lista = json.loads(res.text)
    except json.decoder.JSONDecodeError:
        return "fail"

    return lista['tier']


def load_json() -> dict:
    if not os.path.exists(file):
        response = requests.get(url)
        steam_ids = json.loads(response.text)["applist"]["apps"]
        ids = {}
        for game in steam_ids:
            ids[game["name"]] = game["appid"]

        with open(file, "w") as f:
            f.write(json.dumps(ids))
            f.close()
        return ids
    else:
        return json.loads(open(file, "r").read())


def get_steam_id(title: str):
    # workarounds for satisfactory
    title = title.replace("Early Access", "").replace("Experimental", "").strip()
    if not os.path.exists(file):
        response = requests.get(url)
        ids = {}
        steam_ids = json.loads(response.text)["applist"]["apps"]
        for game in steam_ids:
            ids[game["name"]] = game["appid"]

        with open(file, "w") as f:
            f.write(json.dumps(ids))
            f.close()
    else:
        ids = json.loads(open(file, "r").read())
    if title in ids.keys():
        steam_name = [title]
    else:
        steam_name = difflib.get_close_matches(title, ids.keys(), n=1)
    if steam_name:
        return ids[steam_name[0]]
    else:
        return 0



def check_time():  # this function check if it's time to update
    global file
    text = open(file, 'r')
    json_table = json.load(text)
    text.close()

    today = date.today()
    day = 0  # it controls how many days it's necessary for an update
    for i in 'ymd':
        if i == 'd':
            day = 7
        else:
            day = 0
        if int(today.strftime('%' + i)) > int(json_table['data'][i]) + day:
            return 1
        else:
            return 0



import os
import platform
import shutil
import subprocess
import sys
from logging import getLogger

from PyQt5.QtCore import QSettings, Qt
from PyQt5.QtWidgets import QFileDialog, QWidget

from rare.components.tabs.settings.rpc_settings import RPCSettings
from rare.ui.components.tabs.settings.rare import Ui_RareSettings
from rare.utils import utils
from rare.utils.extra_widgets import PathEdit
from rare.utils.utils import get_lang, get_possible_langs, get_color_schemes, get_style_sheets

logger = getLogger("RareSettings")

languages = [
    ("en", "English"),
    ("de", "Deutsch"),
    ("fr", "Français")
]


class RareSettings(QWidget, Ui_RareSettings):
    def __init__(self):
        super(RareSettings, self).__init__()
        self.setupUi(self)

        # (widget_name, option_name, default)
        self.checkboxes = [
            (self.sys_tray, "sys_tray", True),
            (self.auto_update, "auto_update", False),
            (self.confirm_start, "confirm_start", False),
            (self.auto_sync_cloud, "auto_sync_cloud", True),
            (self.notification, "notification", True),
            (self.save_size, "save_size", False),
            (self.log_games, "show_console", False)
        ]

        self.settings = QSettings()
        self.img_dir_path = self.settings.value("img_dir", os.path.expanduser("~/.cache/rare/images/"), type=str)
        language = self.settings.value("language", get_lang(), type=str)
        self.logdir = os.path.expanduser("~/.cache/rare/logs")

        # Select Image directory
        self.img_dir = PathEdit(self.img_dir_path, file_type=QFileDialog.DirectoryOnly, save_func=self.save_path)
        self.img_dir_layout.addWidget(self.img_dir)

        # Select lang
        self.lang_select.addItems([i[1] for i in languages])
        if language in get_possible_langs():
            index = [lang[0] for lang in languages].index(language)
            self.lang_select.setCurrentIndex(index)
        else:
            self.lang_select.setCurrentIndex(0)
        self.lang_select.currentIndexChanged.connect(self.update_lang)

        colors = get_color_schemes()
        self.color_select.addItems(colors)
        if (color := self.settings.value("color_scheme")) in colors:
            self.color_select.setCurrentIndex(self.color_select.findText(color))
            self.color_select.setDisabled(False)
            self.style_select.setDisabled(True)
        else:
            self.color_select.setCurrentIndex(0)
        self.color_select.currentIndexChanged.connect(self.on_color_select_changed)

        styles = get_style_sheets()
        self.style_select.addItems(styles)
        if (style := self.settings.value("style_sheet")) in styles:
            self.style_select.setCurrentIndex(self.style_select.findText(style))
            self.style_select.setDisabled(False)
            self.color_select.setDisabled(True)
        else:
            self.style_select.setCurrentIndex(0)
        self.style_select.currentIndexChanged.connect(self.on_style_select_changed)

        self.interface_info.setVisible(False)

        self.rpc = RPCSettings()
        self.rpc_layout.addWidget(self.rpc, alignment=Qt.AlignTop)

        for cb in self.checkboxes:
            widget, option, default = cb
            widget.setChecked(self.settings.value(option, default, bool))
            widget.stateChanged.connect(
                lambda: self.settings.setValue(option, widget.isChecked())
            )

        if platform.system() == "Linux":
            self.desktop_file = os.path.expanduser("~/Desktop/Rare.desktop")
            self.start_menu_link = os.path.expanduser("~/.local/share/applications/Rare.desktop")
        elif platform.system() == "Windows":
            self.desktop_file = os.path.expanduser("~/Desktop/Rare.lnk")
            self.start_menu_link = os.path.expandvars("%appdata%/Microsoft/Windows/Start Menu")
        else:
            self.desktop_file = ""
            self.start_menu_link = ""

        if os.path.exists(self.desktop_file):
            self.desktop_link.setText(self.tr("Remove desktop link"))
        if os.path.exists(self.start_menu_link):
            self.startmenu_link.setText(self.tr("Remove start menu link"))

        self.desktop_link.clicked.connect(self.create_desktop_link)
        self.startmenu_link.clicked.connect(self.create_start_menu_link)

        self.log_dir_open_button.clicked.connect(self.open_dir)
        self.log_dir_clean_button.clicked.connect(self.clean_logdir)

        logdir = os.path.expanduser("~/.cache/rare/logs")
        # get size of logdir
        size = 0
        for i in os.listdir(logdir):
            size += os.path.getsize(os.path.join(logdir, i))

        self.log_dir_size_label.setText(utils.get_size(size))
        # self.log_dir_clean_button.setVisible(False)
        # self.log_dir_size_label.setVisible(False)

    def clean_logdir(self):
        for i in os.listdir(os.path.expanduser("~/.cache/rare/logs")):
            os.remove(os.path.expanduser("~/.cache/rare/logs/") + i)
        self.log_dir_size_label.setText("0KB")

    def create_start_menu_link(self):
        if not os.path.exists(self.start_menu_link):
            utils.create_rare_desktop_link("start_menu")
            self.startmenu_link.setText(self.tr("Remove start menu link"))
        else:
            os.remove(self.start_menu_link)
            self.startmenu_link.setText(self.tr("Create start menu link"))

    def create_desktop_link(self):
        if not os.path.exists(self.desktop_file):
            utils.create_rare_desktop_link("desktop")
            self.desktop_link.setText(self.tr("Remove Desktop link"))
        else:
            os.remove(self.desktop_file)
            self.desktop_link.setText(self.tr("Create desktop link"))

    def on_color_select_changed(self, color):
        if color:
            self.style_select.setCurrentIndex(0)
            self.style_select.setDisabled(True)
            self.settings.setValue("color_scheme", self.color_select.currentText())
        else:
            self.settings.setValue("color_scheme", "")
            self.style_select.setDisabled(False)
        self.interface_info.setVisible(True)

    def on_style_select_changed(self, style):
        if style:
            self.color_select.setCurrentIndex(0)
            self.color_select.setDisabled(True)
            self.settings.setValue("style_sheet", self.style_select.currentText())
        else:
            self.settings.setValue("style_sheet", "")
            self.color_select.setDisabled(False)
        self.interface_info.setVisible(True)

    def open_dir(self):
        if platform.system() == "Windows":
            os.startfile(self.logdir)
        else:
            opener = "open" if sys.platform == "darwin" else "xdg-open"
            subprocess.Popen([opener, self.logdir])

    def save_window_size(self):
        self.settings.setValue("save_size", self.save_size.isChecked())
        self.settings.remove("window_size")

    def save_path(self):
        self.update_path()

    def update_lang(self, i: int):
        self.settings.setValue("language", languages[i][0])
        self.interface_info.setVisible(True)

    def update_path(self):
        old_path = self.img_dir_path
        new_path = self.img_dir.text()

        if old_path != new_path:
            if not os.path.exists(new_path):
                os.makedirs(new_path)
            elif len(os.listdir(new_path)) > 0:
                logger.warning("New directory is not empty")
                return
            logger.info("Move Images")
            for i in os.listdir(old_path):
                try:
                    shutil.move(os.path.join(old_path, i), os.path.join(new_path, i))
                except:
                    pass
            os.rmdir(old_path)
            self.img_dir_path = new_path
            self.settings.setValue("img_dir", new_path)



import json
from logging import getLogger

from PyQt5.QtCore import pyqtSignal, QUrl
from PyQt5.QtGui import QDesktopServices
from PyQt5.QtWidgets import QWidget

from custom_legendary.core import LegendaryCore
from rare.ui.components.dialogs.login.browser_login import Ui_BrowserLogin

logger = getLogger("BrowserLogin")


class BrowserLogin(QWidget, Ui_BrowserLogin):
    success = pyqtSignal()
    changed = pyqtSignal()
    login_url = "https://www.epicgames.com/id/login?redirectUrl=https%3A%2F%2Fwww.epicgames.com%2Fid%2Fapi%2Fredirect"

    def __init__(self, core: LegendaryCore, parent=None):
        super(BrowserLogin, self).__init__(parent=parent)
        self.setupUi(self)

        self.core = core

        self.open_button.clicked.connect(self.open_browser)
        self.sid_edit.textChanged.connect(self.changed.emit)

    def is_valid(self):
        return len(self.sid_edit.text()) == 32

    def do_login(self):
        self.status_label.setText(self.tr("Logging in..."))
        sid = self.sid_edit.text()
        # when the text copied
        if sid.startswith("{") and sid.endswith("}"):
            sid = json.loads(sid)["sid"]
        try:
            token = self.core.auth_sid(sid)
            if self.core.auth_code(token):
                logger.info(f"Successfully logged in as {self.core.lgd.userdata['displayName']}")
                self.success.emit()
            else:
                self.status_label.setText(self.tr("Login failed."))
                logger.warning("Failed to login through browser")
        except Exception as e:
            logger.warning(e)

    def open_browser(self):
        QDesktopServices.openUrl(QUrl(self.login_url))
